class PhExecutionModes:
    USER = 1
    DEV = 2
    KNOWN_ISSUES = 22
    UNIT_TESTING = 3
    SAMPLE_SPECIFIC = 4
    SAMPLE_GENERIC = 5
    ALL = 6
    UNIT_TESTING_EXTERNAL = 7
    SAMPLES_LIST = 8
    CLI = 9
    PJ_EXCLUSIVE = 999

    KEYS_NAME = {
        USER: 'user',
        DEV: 'dev',
        KNOWN_ISSUES: 'known_issues',
        UNIT_TESTING: 'unit_testing',
        SAMPLE_SPECIFIC: 'sample_specific',
        SAMPLE_GENERIC: 'sample_generic',
        ALL: 'all',
        UNIT_TESTING_EXTERNAL: 'unit_testing_external',
        SAMPLES_LIST: 'samples_list',
        CLI: 'cli',
        PJ_EXCLUSIVE: 'pj_exclusive',
    }

    @classmethod
    def get_key_name(cls, key):
        return cls.KEYS_NAME.get(key, key)



from play_helpers._common import _PhCommon
from play_helpers.ph_constants import PhConstants


def get_exception_object(e):
    return e.args[0] if len(e.args) > 0 else e


class PhExceptionHelper:
    def __init__(self, msg_key=None, function_name=None, line_num=None, exception=None, msg_value=None,
                 key_value_sep=PhConstants.SEPERATOR_KEY_VALUE, additional_msgs_list=None, summary_msg=None,
                 known=None):
        self.msg_key = msg_key
        self.msg_value = str(msg_value) if msg_value else None
        self.key_value_sep = key_value_sep
        self.summary_msg = summary_msg
        self.known = known
        if additional_msgs_list:
            if not isinstance(additional_msgs_list, list):
                additional_msgs_list = [additional_msgs_list]
        self.additional_msgs_list = additional_msgs_list
        self.function_name = function_name
        self.line_num = line_num
        self.exception = exception
        self.exception_str = str(self.exception) if self.exception else None
        #
        self.msg = None
        self.__set_exception()
        self.details = None
        self.__set_details()

    def __set_exception(self):
        self.msg = self.key_value_sep.join(filter(None, [self.msg_key, self.msg_value]))

    def get_exception(self):
        return self.exception(self.msg)

    def set_summary_msg(self, msg, known=False):
        if msg is not None:
            self.summary_msg = msg
            self.known = known
            self.__set_details()

    def get_details(self):
        msg = PhConstants.SEPERATOR_TWO_WORDS.join(
            [PhConstants.KNOWN if self.known else PhConstants.UNKNOWN, self.details])
        return msg

    def __set_details(self):
        exception_summary = _PhCommon.get_key_value_pair(PhConstants.SUMMARY, self.summary_msg, pair_is_must=True)
        exception_details = _PhCommon.get_key_value_pair(PhConstants.DETAILS, PhConstants.SEPERATOR_MULTI_OBJ.join(
            filter(None, [self.exception_str, self.msg])))
        exception_additional_data = PhConstants.SEPERATOR_MULTI_OBJ.join(
            filter(None, self.additional_msgs_list)) if self.additional_msgs_list else None
        exception_header = _PhCommon.get_key_value_pair(PhConstants.EXCEPTION_OCCURRED_AT_FUNC,
                                                        self.function_name) if self.function_name else PhConstants.EXCEPTION_OCCURRED
        self.details = PhConstants.SEPERATOR_MULTI_OBJ.join(
            filter(None, [exception_header, exception_summary, exception_details, exception_additional_data]))



from play_helpers.ph_exception_helper import PhExceptionHelper


class PhException(Exception):
    def __init__(self, msg):
        self.msg = msg
        super().__init__(msg)

    def __str__(self):
        return PhExceptionHelper(msg=self.msg)



import binascii
import hashlib
from itertools import cycle

from Crypto.Cipher import AES

from play_helpers.ph_defaults import PhDefaults
from play_helpers.ph_util import PhUtil


class PhCrypto:

    @classmethod
    def hash_algos_list(cls):
        return list(sorted(hashlib.algorithms_available))

    @classmethod
    def hash_str(cls, msg, hash_algo='sha256', hex_digest=True,
                 encoding=PhDefaults.CHAR_ENCODING, encoding_errors=PhDefaults.CHAR_ENCODING_ERRORS):
        """
        :param msg:
        :param hash_algo: check hash_algos_list()

        'md5-sha1', 'sha3_224', 'sha3_256', 'shake_128', 'sm3', 'sha512', 'sha3_384', 'sha256', 'sha384',
        'whirlpool', 'sha3_512', 'mdc2', 'sha1', 'md4', 'sha512_256', 'ripemd160', 'blake2s', 'blake2b',
        'sha512_224', 'md5', 'shake_256', 'sha224'
        :param hex_digest:
        :param encoding:
        :param encoding_errors:

        :return:
        """
        hash_obj = hashlib.new(hash_algo)
        hash_obj.update(msg.encode(encoding=encoding, errors=encoding_errors))
        return hash_obj.hexdigest() if hex_digest else hash_obj.digest()

    @classmethod
    def hash_str_sha256(cls, msg, hex_digest=True, encoding=PhDefaults.CHAR_ENCODING,
                        encoding_errors=PhDefaults.CHAR_ENCODING_ERRORS):
        """

        :param msg:
        :param hex_digest:
        :param encoding:
        :param encoding_errors:

        :return:
        """
        sha256 = hashlib.sha256()
        sha256.update(msg.encode(encoding=encoding, errors=encoding_errors))
        return sha256.hexdigest() if hex_digest else sha256.digest()

    @classmethod
    def hash_file(cls, file_obj, hex_digest=True):
        """

        :param file_obj:
        :param hex_digest:
        :param msg:
        :return:
        """
        # A arbitrary (but fixed) buffer size (change accordingly)
        # 65536 = 65536 bytes = 64 kilobytes
        BUF_SIZE = 65536
        # Initializing the sha256() method
        sha256 = hashlib.sha256()

        # Opening the file provided as the first commandline argument
        with open(file_obj, 'rb') as f:
            while True:
                # reading data = BUF_SIZE from the file and saving it in a variable
                data = f.read(BUF_SIZE)
                # True if eof = 1
                if not data:
                    break
                # Passing that data to that sh256 hash function (updating the function with that data)
                sha256.update(data)
        # sha256.hexdigest() hashes all the input data passed to the sha256() via sha256.update()
        # Acts as a finalize method, after which all the input data gets hashed hexdigest()
        # hashes the data, and returns the output in hexadecimal format
        return sha256.hexdigest() if hex_digest else sha256.digest()

    @classmethod
    def logical_xor_(cls, hex_str_msg, hex_str_key):
        """
        Logical XOR among two hex String
        :param hex_str_msg: Main Message as Hex Str
        :param hex_str_key: Key as Hex Str
        :return: output as hex String
        """
        # Convert to Binary
        bin_str_msg = bytearray.fromhex(hex_str_msg)
        bin_str_key = bytearray.fromhex(hex_str_key)
        data = bytearray()
        # Cycle of key is needed for scenario when len(data) > len(key)
        for c, k in zip(bin_str_msg, cycle(bin_str_key)):
            data.append(c ^ k)
        data = data.hex().upper()
        # print('data' + data)
        return data

    @classmethod
    def logical_xor(cls, str1_hex, str2_hex):
        """

        :param str1_hex:
        :param str2_hex:
        :return:
        """
        if len(str1_hex) != len(str2_hex):
            return False
        data = bytearray(
            [x ^ y for x, y in zip(bytearray.fromhex(str1_hex), bytearray.fromhex(str2_hex))]).hex().upper()
        return data

    @classmethod
    def aes_encrypt(cls, hex_strkey, hex_str_buff, algo=AES.MODE_CBC, hex_str_iv='00000000000000000000000000000000'):
        """
        AES Encryption
        :param hex_strkey: Key as Hex Str
        :param hex_str_buff: Main Message as Hex Str to be encrypted
        :param algo: algo to be used
        :param hex_str_iv: iv value as Hex Str
        :return: output as hex String
        """
        encryptor = AES.new(binascii.unhexlify(hex_strkey), algo, binascii.unhexlify(hex_str_iv))
        data = encryptor.encrypt(binascii.unhexlify(hex_str_buff))
        data = data.hex().upper()
        # print('data' + data)
        return data

    @classmethod
    def generate_opc(cls, hex_str_ki, hex_str_op):
        """
        Generate Opc using Ki and Op
        :param hex_str_ki: Ki as hex String
        :param hex_str_op: OP as hex String
        :return: hexStrOpc: OPC as hex String
        """
        hex_str_opc = cls.aes_encrypt(hex_str_ki, hex_str_op)
        return cls.logical_xor_(hex_str_opc, hex_str_op)

    @classmethod
    def gen_opc(cls, ki_hex, op_hex):
        """

        :param ki_hex:
        :param op_hex:
        :return:
        """
        iv = '00000000000000000000000000000000'
        opc_hex = binascii.hexlify(AES.new(binascii.unhexlify(ki_hex), AES.MODE_CBC, IV=binascii.unhexlify(iv)).encrypt(
            binascii.unhexlify(op_hex))).decode()
        # print(opc_hex)
        return cls.logical_xor(opc_hex, op_hex)

    @classmethod
    def sum_to_single_digit(cls, num):
        return int(num) if (num < 10) else int(cls.sum_to_single_digit(num / 10) + num % 10)

    @classmethod
    def get_odd_even_digits(cls, msg, sum_to_single_digit=True):
        digits = list(msg)
        odd_digits_rev = [*map(int, digits[-2::-2])]
        even_digits_rev = [
            *map(lambda x: cls.sum_to_single_digit(int(x) * 2), digits[-1::-2])] if sum_to_single_digit else [
            *map(int, digits[-1::-2])]
        return odd_digits_rev, even_digits_rev

    @classmethod
    def get_luhn_digit(cls, msg):
        """
	    TODO: Function needs to be thoroughly tested

        :param msg:
        :return:
        """
        try:
            odd_digits_rev, even_digits_rev = cls.get_odd_even_digits(msg, sum_to_single_digit=True)
            x = sum(odd_digits_rev) + sum(even_digits_rev)
            x = (10 - x % 10)
            return 0 if x == 10 else x
        except ValueError:
            return None
        except IndexError:
            return None

    @classmethod
    def validate_luhn_digit(cls, msg):
        """
        TODO:Check if iccid last digit is with or without Luhn Check sum digit
        :param msg:
        :return:
        """
        msg = PhUtil.trim_and_kill_all_white_spaces(msg)
        check_sum_length = 1
        if not msg:
            return False
        if not PhUtil.is_numeric(msg):
            return False
        last_digit = int(msg[-check_sum_length:])
        luhn = cls.get_luhn_digit(msg[:-check_sum_length])
        return luhn == last_digit

    @classmethod
    def validate_eid(cls, msg):
        """
        TODO:Check if EID last digit is with or without Luhn Check sum digit
        :param msg:
        :return:
        """
        msg = PhUtil.trim_and_kill_all_white_spaces(msg)
        if not msg:
            return False
        if not PhUtil.is_numeric(msg):
            return False
        msg_int = int(msg)
        reminder = msg_int % 97
        return reminder == 1

    @classmethod
    def append_luhn(cls, str_plain_data, min_len_iccid_wo_luhn=18):
        return cls.append_luhn_if_needed(str_plain_data, len(str_plain_data))

    @classmethod
    def append_luhn_if_needed(cls, str_plain_data, min_len_iccid_wo_luhn=18):
        """
        Consider the case:
            89446172120000011  : 17 Chars, Luhn digit 4
            894461721200000114 : 18 Chars, Luhn digit 6

        :param str_plain_data:
        :param min_len_iccid_wo_luhn:
        :return:
        """
        if (len(str_plain_data) == min_len_iccid_wo_luhn) or (not (cls.validate_luhn_digit(str_plain_data))):
            # luhn has to be added, as length is minimum
            # Lugn Digit is not Present
            str_plain_data = str_plain_data + str(cls.get_luhn_digit(str_plain_data))
        return str_plain_data

    @classmethod
    def remove_luhn_if_present(cls, str_plain_data):
        """

        :param str_plain_data:
        :return:
        """
        if cls.validate_luhn_digit(str_plain_data):  # Lugn Digit is Present
            str_plain_data = str_plain_data[0:-1]
        return str_plain_data

    @classmethod
    def compare_two_files(cls, file_1, file_2):
        """

        :param file_1:
        :param file_2:
        :return:
        """
        # TODO: Validate it, using getting generated files?
        # Ref: https://www.geeksforgeeks.org/compare-two-files-using-hashing-in-python/
        # Ref: https://docs.python.org/3/library/hashlib.html#file-hashing
        f1_hash = cls.hash_file(file_1)
        f2_hash = cls.hash_file(file_2)
        # Doing primitive string comparison to check whether the two hashes match or not
        if f1_hash == f2_hash:
            print('Both files are same')
            print(f'Hash: {f1_hash}')

        else:
            print('Files are different!')
            print(f'Hash of 1st File: {f1_hash}')
            print(f'Hash of 2nd File: {f2_hash}')



from play_helpers.ph_constants import PhConstants
from play_helpers.ph_exception_helper import PhExceptionHelper


class PhMasterDataKeys:
    DATA = 'data'
    META_DATA = 'meta_data'
    INFO_DATA = 'info_data'
    ERROR_DATA = 'error_data'


class PhMasterData:
    """

    """

    def __init__(self, data=None, meta_data=None, info_data=None, error_data=None):
        """

        :param data:
        :param meta_data:
        :param info_data:
        :param error_data:
        """
        self.master_data = {}
        if data is not None:
            self.master_data.update({PhMasterDataKeys.DATA: data})
        if meta_data is not None:
            self.master_data.update({PhMasterDataKeys.META_DATA: meta_data})
        if info_data is not None:
            self.master_data.update({PhMasterDataKeys.INFO_DATA: info_data})
        if error_data is not None:
            self.master_data.update({PhMasterDataKeys.ERROR_DATA: error_data})

    def set_master_data(self, key, value):
        if key in self.master_data:
            self.master_data[key] = value
        else:
            self.master_data.update({key: value})

    def get_master_data(self, key):
        return self.master_data.get(key, None)

    def get(self):
        return self.master_data

    def get_output_data(self, only_output):
        output_data = PhConstants.STR_EMPTY
        info_data = PhConstants.STR_EMPTY
        meta_data = self.get_master_data(PhMasterDataKeys.META_DATA)
        if meta_data is not None:
            #  MetaData Object is Present
            # if isinstance(meta_data, MetaData):
            output_data = meta_data.get_parsed_data()
            info_data = meta_data.get_info_data()
        error_data = self.get_master_data(PhMasterDataKeys.ERROR_DATA)
        if error_data is not None:
            # ErrorData Object is Present
            output_data = error_data.get_details() if isinstance(error_data, PhExceptionHelper) else error_data
        return output_data if only_output else (output_data, info_data)



from play_helpers.ph_constants import PhConstants
from play_helpers.ph_file_extensions import PhFileExtensions
from play_helpers.ph_modes_error_handling import PhErrorHandlingModes
from play_helpers.ph_modes_execution import PhExecutionModes


class PhDefaults:
    #############
    # Version Objects
    #############
    DUMMY_VERSION = '-1.-1.-1'
    VERSION = '1.0.1'
    #############
    # Generic Objects
    #############
    CHAR_ENCODING = PhConstants.CHAR_ENCODING_UTF8
    CHAR_ENCODING_ERRORS = PhConstants.CHAR_ENCODING_ERRORS_REPLACE
    EXECUTION_MODE = PhExecutionModes.USER
    ERROR_HANDLING_MODE = PhErrorHandlingModes.CONTINUE_ON_ERROR
    ARCHIVE_FORMAT = PhFileExtensions.ZIP
    #############
    # Data Objects
    #############
    # Common Objects
    # INPUT_DATA
    PRINT_INPUT = True
    PRINT_OUTPUT = True
    PRINT_INFO = True
    QUITE_MODE = False
    # REMARKS
    ENCODING = CHAR_ENCODING
    ENCODING_ERRORS = CHAR_ENCODING_ERRORS
    OUTPUT_PATH = ''
    OUTPUT_FILE_NAME_KEYWORD = ''
    ARCHIVE_OUTPUT = True
    ARCHIVE_OUTPUT_FORMAT = ARCHIVE_FORMAT


class PhDefaultTypesInclude:
    # Common Objects
    # INPUT_DATA
    PRINT_INPUT = bool
    PRINT_OUTPUT = bool
    PRINT_INFO = bool
    QUITE_MODE = bool
    # REMARKS
    ENCODING = str
    ENCODING_ERRORS = str
    OUTPUT_PATH = str
    ARCHIVE_OUTPUT = bool
    ARCHIVE_OUTPUT_FORMAT = str


class PhDefaultTypesExclude:
    # Common Objects
    INPUT_DATA = [int, float, bool]



import time

from play_helpers.ph_keys import PhKeys
from play_helpers.ph_util import PhUtil


def _get_time():
    """
    :return: Script Time, CPU Time
    """
    script_time = time.time()
    cpu_time = time.process_time()
    return script_time, cpu_time


def _diff_time(start_time, end_time):
    return end_time - start_time


class TimeStates:
    TIMER_INIT = 1
    TIMER_ON = 2
    TIMER_OFF = 3

    name_mapping = {
        TIMER_INIT: 'TIMER_INIT',
        TIMER_ON: 'TIMER_ON',
        TIMER_OFF: 'TIMER_OFF',
    }


class PhTime:
    """

    """

    def __init__(self):
        """

        """
        self.start_time_script = None
        self.start_time_cpu = None
        self.end_time_script = None
        self.end_time_cpu = None
        self.diff_script = None
        self.diff_cpu = None
        self.state = TimeStates.TIMER_INIT

    def start(self):
        """

        :return:
        """
        self.start_time_script, self.start_time_cpu = _get_time()
        self.state = TimeStates.TIMER_ON

    def stop(self):
        """

        :return:
        """
        self.end_time_script, self.end_time_cpu = _get_time()
        self.state = TimeStates.TIMER_OFF

    def get_state(self):
        return TimeStates.name_mapping.get(self.state, PhKeys.UNKNOWN)

    def result(self):
        end_time_script = self.end_time_script
        end_time_cpu = self.end_time_cpu
        if self.state == TimeStates.TIMER_ON:
            end_time_script, end_time_cpu = _get_time()
        self.diff_script = _diff_time(start_time=self.start_time_script, end_time=end_time_script)
        self.diff_cpu = _diff_time(start_time=self.start_time_cpu, end_time=end_time_cpu)

    def print(self):
        """

        :return:
        """
        self.result()
        PhUtil.print_separator(main_text='Time Details')
        print(f'Current Time Stamp: {PhUtil.get_time_stamp(default_format=True)}')
        print(f'Execution Time: {PhUtil.format_time(self.diff_script, time_interval=True)}; '
              f'Milliseconds: {self.diff_script * 1000}')
        print(f'Execution Time CPU: {PhUtil.format_time(self.diff_cpu, time_interval=True)}; '
              f'Milliseconds: {self.diff_cpu * 1000}')
        print(f'Timer State:', self.get_state())
        PhUtil.print_separator()



from collections import namedtuple, OrderedDict

"""
Ref: https://github.com/MMVonnSeek/Python-Color-Constants-Module/tree/main

Also Check (TODO):
    D:\Other\Github_Self\qrPlay\venv\Lib\site-packages\PIL\ImageColor.py
    
    colormap: dict[str, str | tuple[int, int, int]] = {
"""


class RGB(namedtuple('RGB', 'red, green, blue')):
    def hex_format(self):
        """

        :return: Returns color in hex format
        """
        return '#{:02X}{:02X}{:02X}'.format(self.red, self.green, self.blue)


class PhColors:
    """

    """

    colors = {}  # dict of colors
    """
    Color Constants
    """
    ALICEBLUE = RGB(240, 248, 255)
    ANTIQUEWHITE = RGB(250, 235, 215)
    ANTIQUEWHITE1 = RGB(255, 239, 219)
    ANTIQUEWHITE2 = RGB(238, 223, 204)
    ANTIQUEWHITE3 = RGB(205, 192, 176)
    ANTIQUEWHITE4 = RGB(139, 131, 120)
    AQUA = RGB(0, 255, 255)
    AQUAMARINE1 = RGB(127, 255, 212)
    AQUAMARINE2 = RGB(118, 238, 198)
    AQUAMARINE3 = RGB(102, 205, 170)
    AQUAMARINE4 = RGB(69, 139, 116)
    AZURE1 = RGB(240, 255, 255)
    AZURE2 = RGB(224, 238, 238)
    AZURE3 = RGB(193, 205, 205)
    AZURE4 = RGB(131, 139, 139)
    BANANA = RGB(227, 207, 87)
    BEIGE = RGB(245, 245, 220)
    BISQUE1 = RGB(255, 228, 196)
    BISQUE2 = RGB(238, 213, 183)
    BISQUE3 = RGB(205, 183, 158)
    BISQUE4 = RGB(139, 125, 107)
    BLACK = RGB(0, 0, 0)
    BLANCHEDALMOND = RGB(255, 235, 205)
    BLUE = RGB(0, 0, 255)
    BLUE2 = RGB(0, 0, 238)
    BLUE3 = RGB(0, 0, 205)
    BLUE4 = RGB(0, 0, 139)
    BLUEVIOLET = RGB(138, 43, 226)
    BRICK = RGB(156, 102, 31)
    BROWN = RGB(165, 42, 42)
    BROWN1 = RGB(255, 64, 64)
    BROWN2 = RGB(238, 59, 59)
    BROWN3 = RGB(205, 51, 51)
    BROWN4 = RGB(139, 35, 35)
    BURLYWOOD = RGB(222, 184, 135)
    BURLYWOOD1 = RGB(255, 211, 155)
    BURLYWOOD2 = RGB(238, 197, 145)
    BURLYWOOD3 = RGB(205, 170, 125)
    BURLYWOOD4 = RGB(139, 115, 85)
    BURNTSIENNA = RGB(138, 54, 15)
    BURNTUMBER = RGB(138, 51, 36)
    CADETBLUE = RGB(95, 158, 160)
    CADETBLUE1 = RGB(152, 245, 255)
    CADETBLUE2 = RGB(142, 229, 238)
    CADETBLUE3 = RGB(122, 197, 205)
    CADETBLUE4 = RGB(83, 134, 139)
    CADMIUMORANGE = RGB(255, 97, 3)
    CADMIUMYELLOW = RGB(255, 153, 18)
    CARROT = RGB(237, 145, 33)
    CHARTREUSE1 = RGB(127, 255, 0)
    CHARTREUSE2 = RGB(118, 238, 0)
    CHARTREUSE3 = RGB(102, 205, 0)
    CHARTREUSE4 = RGB(69, 139, 0)
    CHOCOLATE = RGB(210, 105, 30)
    CHOCOLATE1 = RGB(255, 127, 36)
    CHOCOLATE2 = RGB(238, 118, 33)
    CHOCOLATE3 = RGB(205, 102, 29)
    CHOCOLATE4 = RGB(139, 69, 19)
    COBALT = RGB(61, 89, 171)
    COBALTGREEN = RGB(61, 145, 64)
    COLDGREY = RGB(128, 138, 135)
    CORAL = RGB(255, 127, 80)
    CORAL1 = RGB(255, 114, 86)
    CORAL2 = RGB(238, 106, 80)
    CORAL3 = RGB(205, 91, 69)
    CORAL4 = RGB(139, 62, 47)
    CORNFLOWERBLUE = RGB(100, 149, 237)
    CORNSILK1 = RGB(255, 248, 220)
    CORNSILK2 = RGB(238, 232, 205)
    CORNSILK3 = RGB(205, 200, 177)
    CORNSILK4 = RGB(139, 136, 120)
    CRIMSON = RGB(220, 20, 60)
    CYAN2 = RGB(0, 238, 238)
    CYAN3 = RGB(0, 205, 205)
    CYAN4 = RGB(0, 139, 139)
    DARKGOLDENROD = RGB(184, 134, 11)
    DARKGOLDENROD1 = RGB(255, 185, 15)
    DARKGOLDENROD2 = RGB(238, 173, 14)
    DARKGOLDENROD3 = RGB(205, 149, 12)
    DARKGOLDENROD4 = RGB(139, 101, 8)
    DARKGRAY = RGB(169, 169, 169)
    DARKGREEN = RGB(0, 100, 0)
    DARKKHAKI = RGB(189, 183, 107)
    DARKOLIVEGREEN = RGB(85, 107, 47)
    DARKOLIVEGREEN1 = RGB(202, 255, 112)
    DARKOLIVEGREEN2 = RGB(188, 238, 104)
    DARKOLIVEGREEN3 = RGB(162, 205, 90)
    DARKOLIVEGREEN4 = RGB(110, 139, 61)
    DARKORANGE = RGB(255, 140, 0)
    DARKORANGE1 = RGB(255, 127, 0)
    DARKORANGE2 = RGB(238, 118, 0)
    DARKORANGE3 = RGB(205, 102, 0)
    DARKORANGE4 = RGB(139, 69, 0)
    DARKORCHID = RGB(153, 50, 204)
    DARKORCHID1 = RGB(191, 62, 255)
    DARKORCHID2 = RGB(178, 58, 238)
    DARKORCHID3 = RGB(154, 50, 205)
    DARKORCHID4 = RGB(104, 34, 139)
    DARKSALMON = RGB(233, 150, 122)
    DARKSEAGREEN = RGB(143, 188, 143)
    DARKSEAGREEN1 = RGB(193, 255, 193)
    DARKSEAGREEN2 = RGB(180, 238, 180)
    DARKSEAGREEN3 = RGB(155, 205, 155)
    DARKSEAGREEN4 = RGB(105, 139, 105)
    DARKSLATEBLUE = RGB(72, 61, 139)
    DARKSLATEGRAY = RGB(47, 79, 79)
    DARKSLATEGRAY1 = RGB(151, 255, 255)
    DARKSLATEGRAY2 = RGB(141, 238, 238)
    DARKSLATEGRAY3 = RGB(121, 205, 205)
    DARKSLATEGRAY4 = RGB(82, 139, 139)
    DARKTURQUOISE = RGB(0, 206, 209)
    DARKVIOLET = RGB(148, 0, 211)
    DEEPPINK1 = RGB(255, 20, 147)
    DEEPPINK2 = RGB(238, 18, 137)
    DEEPPINK3 = RGB(205, 16, 118)
    DEEPPINK4 = RGB(139, 10, 80)
    DEEPSKYBLUE1 = RGB(0, 191, 255)
    DEEPSKYBLUE2 = RGB(0, 178, 238)
    DEEPSKYBLUE3 = RGB(0, 154, 205)
    DEEPSKYBLUE4 = RGB(0, 104, 139)
    DIMGRAY = RGB(105, 105, 105)
    DIMGRAY = RGB(105, 105, 105)
    DODGERBLUE1 = RGB(30, 144, 255)
    DODGERBLUE2 = RGB(28, 134, 238)
    DODGERBLUE3 = RGB(24, 116, 205)
    DODGERBLUE4 = RGB(16, 78, 139)
    EGGSHELL = RGB(252, 230, 201)
    EMERALDGREEN = RGB(0, 201, 87)
    FIREBRICK = RGB(178, 34, 34)
    FIREBRICK1 = RGB(255, 48, 48)
    FIREBRICK2 = RGB(238, 44, 44)
    FIREBRICK3 = RGB(205, 38, 38)
    FIREBRICK4 = RGB(139, 26, 26)
    FLESH = RGB(255, 125, 64)
    FLORALWHITE = RGB(255, 250, 240)
    FORESTGREEN = RGB(34, 139, 34)
    GAINSBORO = RGB(220, 220, 220)
    GHOSTWHITE = RGB(248, 248, 255)
    GOLD1 = RGB(255, 215, 0)
    GOLD2 = RGB(238, 201, 0)
    GOLD3 = RGB(205, 173, 0)
    GOLD4 = RGB(139, 117, 0)
    GOLDENROD = RGB(218, 165, 32)
    GOLDENROD1 = RGB(255, 193, 37)
    GOLDENROD2 = RGB(238, 180, 34)
    GOLDENROD3 = RGB(205, 155, 29)
    GOLDENROD4 = RGB(139, 105, 20)
    GRAY = RGB(128, 128, 128)
    GRAY1 = RGB(3, 3, 3)
    GRAY10 = RGB(26, 26, 26)
    GRAY11 = RGB(28, 28, 28)
    GRAY12 = RGB(31, 31, 31)
    GRAY13 = RGB(33, 33, 33)
    GRAY14 = RGB(36, 36, 36)
    GRAY15 = RGB(38, 38, 38)
    GRAY16 = RGB(41, 41, 41)
    GRAY17 = RGB(43, 43, 43)
    GRAY18 = RGB(46, 46, 46)
    GRAY19 = RGB(48, 48, 48)
    GRAY2 = RGB(5, 5, 5)
    GRAY20 = RGB(51, 51, 51)
    GRAY21 = RGB(54, 54, 54)
    GRAY22 = RGB(56, 56, 56)
    GRAY23 = RGB(59, 59, 59)
    GRAY24 = RGB(61, 61, 61)
    GRAY25 = RGB(64, 64, 64)
    GRAY26 = RGB(66, 66, 66)
    GRAY27 = RGB(69, 69, 69)
    GRAY28 = RGB(71, 71, 71)
    GRAY29 = RGB(74, 74, 74)
    GRAY3 = RGB(8, 8, 8)
    GRAY30 = RGB(77, 77, 77)
    GRAY31 = RGB(79, 79, 79)
    GRAY32 = RGB(82, 82, 82)
    GRAY33 = RGB(84, 84, 84)
    GRAY34 = RGB(87, 87, 87)
    GRAY35 = RGB(89, 89, 89)
    GRAY36 = RGB(92, 92, 92)
    GRAY37 = RGB(94, 94, 94)
    GRAY38 = RGB(97, 97, 97)
    GRAY39 = RGB(99, 99, 99)
    GRAY4 = RGB(10, 10, 10)
    GRAY40 = RGB(102, 102, 102)
    GRAY42 = RGB(107, 107, 107)
    GRAY43 = RGB(110, 110, 110)
    GRAY44 = RGB(112, 112, 112)
    GRAY45 = RGB(115, 115, 115)
    GRAY46 = RGB(117, 117, 117)
    GRAY47 = RGB(120, 120, 120)
    GRAY48 = RGB(122, 122, 122)
    GRAY49 = RGB(125, 125, 125)
    GRAY5 = RGB(13, 13, 13)
    GRAY50 = RGB(127, 127, 127)
    GRAY51 = RGB(130, 130, 130)
    GRAY52 = RGB(133, 133, 133)
    GRAY53 = RGB(135, 135, 135)
    GRAY54 = RGB(138, 138, 138)
    GRAY55 = RGB(140, 140, 140)
    GRAY56 = RGB(143, 143, 143)
    GRAY57 = RGB(145, 145, 145)
    GRAY58 = RGB(148, 148, 148)
    GRAY59 = RGB(150, 150, 150)
    GRAY6 = RGB(15, 15, 15)
    GRAY60 = RGB(153, 153, 153)
    GRAY61 = RGB(156, 156, 156)
    GRAY62 = RGB(158, 158, 158)
    GRAY63 = RGB(161, 161, 161)
    GRAY64 = RGB(163, 163, 163)
    GRAY65 = RGB(166, 166, 166)
    GRAY66 = RGB(168, 168, 168)
    GRAY67 = RGB(171, 171, 171)
    GRAY68 = RGB(173, 173, 173)
    GRAY69 = RGB(176, 176, 176)
    GRAY7 = RGB(18, 18, 18)
    GRAY70 = RGB(179, 179, 179)
    GRAY71 = RGB(181, 181, 181)
    GRAY72 = RGB(184, 184, 184)
    GRAY73 = RGB(186, 186, 186)
    GRAY74 = RGB(189, 189, 189)
    GRAY75 = RGB(191, 191, 191)
    GRAY76 = RGB(194, 194, 194)
    GRAY77 = RGB(196, 196, 196)
    GRAY78 = RGB(199, 199, 199)
    GRAY79 = RGB(201, 201, 201)
    GRAY8 = RGB(20, 20, 20)
    GRAY80 = RGB(204, 204, 204)
    GRAY81 = RGB(207, 207, 207)
    GRAY82 = RGB(209, 209, 209)
    GRAY83 = RGB(212, 212, 212)
    GRAY84 = RGB(214, 214, 214)
    GRAY85 = RGB(217, 217, 217)
    GRAY86 = RGB(219, 219, 219)
    GRAY87 = RGB(222, 222, 222)
    GRAY88 = RGB(224, 224, 224)
    GRAY89 = RGB(227, 227, 227)
    GRAY9 = RGB(23, 23, 23)
    GRAY90 = RGB(229, 229, 229)
    GRAY91 = RGB(232, 232, 232)
    GRAY92 = RGB(235, 235, 235)
    GRAY93 = RGB(237, 237, 237)
    GRAY94 = RGB(240, 240, 240)
    GRAY95 = RGB(242, 242, 242)
    GRAY97 = RGB(247, 247, 247)
    GRAY98 = RGB(250, 250, 250)
    GRAY99 = RGB(252, 252, 252)
    GREEN = RGB(0, 128, 0)
    GREEN1 = RGB(0, 255, 0)
    GREEN2 = RGB(0, 238, 0)
    GREEN3 = RGB(0, 205, 0)
    GREEN4 = RGB(0, 139, 0)
    GREENYELLOW = RGB(173, 255, 47)
    HONEYDEW1 = RGB(240, 255, 240)
    HONEYDEW2 = RGB(224, 238, 224)
    HONEYDEW3 = RGB(193, 205, 193)
    HONEYDEW4 = RGB(131, 139, 131)
    HOTPINK = RGB(255, 105, 180)
    HOTPINK1 = RGB(255, 110, 180)
    HOTPINK2 = RGB(238, 106, 167)
    HOTPINK3 = RGB(205, 96, 144)
    HOTPINK4 = RGB(139, 58, 98)
    INDIANRED = RGB(176, 23, 31)
    INDIANRED = RGB(205, 92, 92)
    INDIANRED1 = RGB(255, 106, 106)
    INDIANRED2 = RGB(238, 99, 99)
    INDIANRED3 = RGB(205, 85, 85)
    INDIANRED4 = RGB(139, 58, 58)
    INDIGO = RGB(75, 0, 130)
    IVORY1 = RGB(255, 255, 240)
    IVORY2 = RGB(238, 238, 224)
    IVORY3 = RGB(205, 205, 193)
    IVORY4 = RGB(139, 139, 131)
    IVORYBLACK = RGB(41, 36, 33)
    KHAKI = RGB(240, 230, 140)
    KHAKI1 = RGB(255, 246, 143)
    KHAKI2 = RGB(238, 230, 133)
    KHAKI3 = RGB(205, 198, 115)
    KHAKI4 = RGB(139, 134, 78)
    LAVENDER = RGB(230, 230, 250)
    LAVENDERBLUSH1 = RGB(255, 240, 245)
    LAVENDERBLUSH2 = RGB(238, 224, 229)
    LAVENDERBLUSH3 = RGB(205, 193, 197)
    LAVENDERBLUSH4 = RGB(139, 131, 134)
    LAWNGREEN = RGB(124, 252, 0)
    LEMONCHIFFON1 = RGB(255, 250, 205)
    LEMONCHIFFON2 = RGB(238, 233, 191)
    LEMONCHIFFON3 = RGB(205, 201, 165)
    LEMONCHIFFON4 = RGB(139, 137, 112)
    LIGHTBLUE = RGB(173, 216, 230)
    LIGHTBLUE1 = RGB(191, 239, 255)
    LIGHTBLUE2 = RGB(178, 223, 238)
    LIGHTBLUE3 = RGB(154, 192, 205)
    LIGHTBLUE4 = RGB(104, 131, 139)
    LIGHTCORAL = RGB(240, 128, 128)
    LIGHTCYAN1 = RGB(224, 255, 255)
    LIGHTCYAN2 = RGB(209, 238, 238)
    LIGHTCYAN3 = RGB(180, 205, 205)
    LIGHTCYAN4 = RGB(122, 139, 139)
    LIGHTGOLDENROD1 = RGB(255, 236, 139)
    LIGHTGOLDENROD2 = RGB(238, 220, 130)
    LIGHTGOLDENROD3 = RGB(205, 190, 112)
    LIGHTGOLDENROD4 = RGB(139, 129, 76)
    LIGHTGOLDENRODYELLOW = RGB(250, 250, 210)
    LIGHTGREY = RGB(211, 211, 211)
    LIGHTPINK = RGB(255, 182, 193)
    LIGHTPINK1 = RGB(255, 174, 185)
    LIGHTPINK2 = RGB(238, 162, 173)
    LIGHTPINK3 = RGB(205, 140, 149)
    LIGHTPINK4 = RGB(139, 95, 101)
    LIGHTSALMON1 = RGB(255, 160, 122)
    LIGHTSALMON2 = RGB(238, 149, 114)
    LIGHTSALMON3 = RGB(205, 129, 98)
    LIGHTSALMON4 = RGB(139, 87, 66)
    LIGHTSEAGREEN = RGB(32, 178, 170)
    LIGHTSKYBLUE = RGB(135, 206, 250)
    LIGHTSKYBLUE1 = RGB(176, 226, 255)
    LIGHTSKYBLUE2 = RGB(164, 211, 238)
    LIGHTSKYBLUE3 = RGB(141, 182, 205)
    LIGHTSKYBLUE4 = RGB(96, 123, 139)
    LIGHTSLATEBLUE = RGB(132, 112, 255)
    LIGHTSLATEGRAY = RGB(119, 136, 153)
    LIGHTSTEELBLUE = RGB(176, 196, 222)
    LIGHTSTEELBLUE1 = RGB(202, 225, 255)
    LIGHTSTEELBLUE2 = RGB(188, 210, 238)
    LIGHTSTEELBLUE3 = RGB(162, 181, 205)
    LIGHTSTEELBLUE4 = RGB(110, 123, 139)
    LIGHTYELLOW1 = RGB(255, 255, 224)
    LIGHTYELLOW2 = RGB(238, 238, 209)
    LIGHTYELLOW3 = RGB(205, 205, 180)
    LIGHTYELLOW4 = RGB(139, 139, 122)
    LIMEGREEN = RGB(50, 205, 50)
    LINEN = RGB(250, 240, 230)
    MAGENTA = RGB(255, 0, 255)
    MAGENTA2 = RGB(238, 0, 238)
    MAGENTA3 = RGB(205, 0, 205)
    MAGENTA4 = RGB(139, 0, 139)
    MANGANESEBLUE = RGB(3, 168, 158)
    MAROON = RGB(128, 0, 0)
    MAROON1 = RGB(255, 52, 179)
    MAROON2 = RGB(238, 48, 167)
    MAROON3 = RGB(205, 41, 144)
    MAROON4 = RGB(139, 28, 98)
    MEDIUMORCHID = RGB(186, 85, 211)
    MEDIUMORCHID1 = RGB(224, 102, 255)
    MEDIUMORCHID2 = RGB(209, 95, 238)
    MEDIUMORCHID3 = RGB(180, 82, 205)
    MEDIUMORCHID4 = RGB(122, 55, 139)
    MEDIUMPURPLE = RGB(147, 112, 219)
    MEDIUMPURPLE1 = RGB(171, 130, 255)
    MEDIUMPURPLE2 = RGB(159, 121, 238)
    MEDIUMPURPLE3 = RGB(137, 104, 205)
    MEDIUMPURPLE4 = RGB(93, 71, 139)
    MEDIUMSEAGREEN = RGB(60, 179, 113)
    MEDIUMSLATEBLUE = RGB(123, 104, 238)
    MEDIUMSPRINGGREEN = RGB(0, 250, 154)
    MEDIUMTURQUOISE = RGB(72, 209, 204)
    MEDIUMVIOLETRED = RGB(199, 21, 133)
    MELON = RGB(227, 168, 105)
    MIDNIGHTBLUE = RGB(25, 25, 112)
    MINT = RGB(189, 252, 201)
    MINTCREAM = RGB(245, 255, 250)
    MISTYROSE1 = RGB(255, 228, 225)
    MISTYROSE2 = RGB(238, 213, 210)
    MISTYROSE3 = RGB(205, 183, 181)
    MISTYROSE4 = RGB(139, 125, 123)
    MOCCASIN = RGB(255, 228, 181)
    NAVAJOWHITE1 = RGB(255, 222, 173)
    NAVAJOWHITE2 = RGB(238, 207, 161)
    NAVAJOWHITE3 = RGB(205, 179, 139)
    NAVAJOWHITE4 = RGB(139, 121, 94)
    NAVY = RGB(0, 0, 128)
    OLDLACE = RGB(253, 245, 230)
    OLIVE = RGB(128, 128, 0)
    OLIVEDRAB = RGB(107, 142, 35)
    OLIVEDRAB1 = RGB(192, 255, 62)
    OLIVEDRAB2 = RGB(179, 238, 58)
    OLIVEDRAB3 = RGB(154, 205, 50)
    OLIVEDRAB4 = RGB(105, 139, 34)
    ORANGE = RGB(255, 128, 0)
    ORANGE1 = RGB(255, 165, 0)
    ORANGE2 = RGB(238, 154, 0)
    ORANGE3 = RGB(205, 133, 0)
    ORANGE4 = RGB(139, 90, 0)
    ORANGERED1 = RGB(255, 69, 0)
    ORANGERED2 = RGB(238, 64, 0)
    ORANGERED3 = RGB(205, 55, 0)
    ORANGERED4 = RGB(139, 37, 0)
    ORCHID = RGB(218, 112, 214)
    ORCHID1 = RGB(255, 131, 250)
    ORCHID2 = RGB(238, 122, 233)
    ORCHID3 = RGB(205, 105, 201)
    ORCHID4 = RGB(139, 71, 137)
    PALEGOLDENROD = RGB(238, 232, 170)
    PALEGREEN = RGB(152, 251, 152)
    PALEGREEN1 = RGB(154, 255, 154)
    PALEGREEN2 = RGB(144, 238, 144)
    PALEGREEN3 = RGB(124, 205, 124)
    PALEGREEN4 = RGB(84, 139, 84)
    PALETURQUOISE1 = RGB(187, 255, 255)
    PALETURQUOISE2 = RGB(174, 238, 238)
    PALETURQUOISE3 = RGB(150, 205, 205)
    PALETURQUOISE4 = RGB(102, 139, 139)
    PALEVIOLETRED = RGB(219, 112, 147)
    PALEVIOLETRED1 = RGB(255, 130, 171)
    PALEVIOLETRED2 = RGB(238, 121, 159)
    PALEVIOLETRED3 = RGB(205, 104, 137)
    PALEVIOLETRED4 = RGB(139, 71, 93)
    PAPAYAWHIP = RGB(255, 239, 213)
    PEACHPUFF1 = RGB(255, 218, 185)
    PEACHPUFF2 = RGB(238, 203, 173)
    PEACHPUFF3 = RGB(205, 175, 149)
    PEACHPUFF4 = RGB(139, 119, 101)
    PEACOCK = RGB(51, 161, 201)
    PINK = RGB(255, 192, 203)
    PINK1 = RGB(255, 181, 197)
    PINK2 = RGB(238, 169, 184)
    PINK3 = RGB(205, 145, 158)
    PINK4 = RGB(139, 99, 108)
    PLUM = RGB(221, 160, 221)
    PLUM1 = RGB(255, 187, 255)
    PLUM2 = RGB(238, 174, 238)
    PLUM3 = RGB(205, 150, 205)
    PLUM4 = RGB(139, 102, 139)
    POWDERBLUE = RGB(176, 224, 230)
    PURPLE = RGB(128, 0, 128)
    PURPLE1 = RGB(155, 48, 255)
    PURPLE2 = RGB(145, 44, 238)
    PURPLE3 = RGB(125, 38, 205)
    PURPLE4 = RGB(85, 26, 139)
    RASPBERRY = RGB(135, 38, 87)
    RAWSIENNA = RGB(199, 97, 20)
    RED = RGB(255, 0, 0)
    RED1 = RGB(255, 0, 0)
    RED2 = RGB(238, 0, 0)
    RED3 = RGB(205, 0, 0)
    RED4 = RGB(139, 0, 0)
    ROSYBROWN = RGB(188, 143, 143)
    ROSYBROWN1 = RGB(255, 193, 193)
    ROSYBROWN2 = RGB(238, 180, 180)
    ROSYBROWN3 = RGB(205, 155, 155)
    ROSYBROWN4 = RGB(139, 105, 105)
    ROYALBLUE = RGB(65, 105, 225)
    ROYALBLUE1 = RGB(72, 118, 255)
    ROYALBLUE2 = RGB(67, 110, 238)
    ROYALBLUE3 = RGB(58, 95, 205)
    ROYALBLUE4 = RGB(39, 64, 139)
    SALMON = RGB(250, 128, 114)
    SALMON1 = RGB(255, 140, 105)
    SALMON2 = RGB(238, 130, 98)
    SALMON3 = RGB(205, 112, 84)
    SALMON4 = RGB(139, 76, 57)
    SANDYBROWN = RGB(244, 164, 96)
    SAPGREEN = RGB(48, 128, 20)
    SEAGREEN1 = RGB(84, 255, 159)
    SEAGREEN2 = RGB(78, 238, 148)
    SEAGREEN3 = RGB(67, 205, 128)
    SEAGREEN4 = RGB(46, 139, 87)
    SEASHELL1 = RGB(255, 245, 238)
    SEASHELL2 = RGB(238, 229, 222)
    SEASHELL3 = RGB(205, 197, 191)
    SEASHELL4 = RGB(139, 134, 130)
    SEPIA = RGB(94, 38, 18)
    SGIBEET = RGB(142, 56, 142)
    SGIBRIGHTGRAY = RGB(197, 193, 170)
    SGICHARTREUSE = RGB(113, 198, 113)
    SGIDARKGRAY = RGB(85, 85, 85)
    SGIGRAY12 = RGB(30, 30, 30)
    SGIGRAY16 = RGB(40, 40, 40)
    SGIGRAY32 = RGB(81, 81, 81)
    SGIGRAY36 = RGB(91, 91, 91)
    SGIGRAY52 = RGB(132, 132, 132)
    SGIGRAY56 = RGB(142, 142, 142)
    SGIGRAY72 = RGB(183, 183, 183)
    SGIGRAY76 = RGB(193, 193, 193)
    SGIGRAY92 = RGB(234, 234, 234)
    SGIGRAY96 = RGB(244, 244, 244)
    SGILIGHTBLUE = RGB(125, 158, 192)
    SGILIGHTGRAY = RGB(170, 170, 170)
    SGIOLIVEDRAB = RGB(142, 142, 56)
    SGISALMON = RGB(198, 113, 113)
    SGISLATEBLUE = RGB(113, 113, 198)
    SGITEAL = RGB(56, 142, 142)
    SIENNA = RGB(160, 82, 45)
    SIENNA1 = RGB(255, 130, 71)
    SIENNA2 = RGB(238, 121, 66)
    SIENNA3 = RGB(205, 104, 57)
    SIENNA4 = RGB(139, 71, 38)
    SILVER = RGB(192, 192, 192)
    SKYBLUE = RGB(135, 206, 235)
    SKYBLUE1 = RGB(135, 206, 255)
    SKYBLUE2 = RGB(126, 192, 238)
    SKYBLUE3 = RGB(108, 166, 205)
    SKYBLUE4 = RGB(74, 112, 139)
    SLATEBLUE = RGB(106, 90, 205)
    SLATEBLUE1 = RGB(131, 111, 255)
    SLATEBLUE2 = RGB(122, 103, 238)
    SLATEBLUE3 = RGB(105, 89, 205)
    SLATEBLUE4 = RGB(71, 60, 139)
    SLATEGRAY = RGB(112, 128, 144)
    SLATEGRAY1 = RGB(198, 226, 255)
    SLATEGRAY2 = RGB(185, 211, 238)
    SLATEGRAY3 = RGB(159, 182, 205)
    SLATEGRAY4 = RGB(108, 123, 139)
    SNOW1 = RGB(255, 250, 250)
    SNOW2 = RGB(238, 233, 233)
    SNOW3 = RGB(205, 201, 201)
    SNOW4 = RGB(139, 137, 137)
    SPRINGGREEN = RGB(0, 255, 127)
    SPRINGGREEN1 = RGB(0, 238, 118)
    SPRINGGREEN2 = RGB(0, 205, 102)
    SPRINGGREEN3 = RGB(0, 139, 69)
    STEELBLUE = RGB(70, 130, 180)
    STEELBLUE1 = RGB(99, 184, 255)
    STEELBLUE2 = RGB(92, 172, 238)
    STEELBLUE3 = RGB(79, 148, 205)
    STEELBLUE4 = RGB(54, 100, 139)
    TAN = RGB(210, 180, 140)
    TAN1 = RGB(255, 165, 79)
    TAN2 = RGB(238, 154, 73)
    TAN3 = RGB(205, 133, 63)
    TAN4 = RGB(139, 90, 43)
    TEAL = RGB(0, 128, 128)
    THISTLE = RGB(216, 191, 216)
    THISTLE1 = RGB(255, 225, 255)
    THISTLE2 = RGB(238, 210, 238)
    THISTLE3 = RGB(205, 181, 205)
    THISTLE4 = RGB(139, 123, 139)
    TOMATO1 = RGB(255, 99, 71)
    TOMATO2 = RGB(238, 92, 66)
    TOMATO3 = RGB(205, 79, 57)
    TOMATO4 = RGB(139, 54, 38)
    TURQUOISE = RGB(64, 224, 208)
    TURQUOISE1 = RGB(0, 245, 255)
    TURQUOISE2 = RGB(0, 229, 238)
    TURQUOISE3 = RGB(0, 197, 205)
    TURQUOISE4 = RGB(0, 134, 139)
    TURQUOISEBLUE = RGB(0, 199, 140)
    VIOLET = RGB(238, 130, 238)
    VIOLETRED = RGB(208, 32, 144)
    VIOLETRED1 = RGB(255, 62, 150)
    VIOLETRED2 = RGB(238, 58, 140)
    VIOLETRED3 = RGB(205, 50, 120)
    VIOLETRED4 = RGB(139, 34, 82)
    WARMGREY = RGB(128, 128, 105)
    WHEAT = RGB(245, 222, 179)
    WHEAT1 = RGB(255, 231, 186)
    WHEAT2 = RGB(238, 216, 174)
    WHEAT3 = RGB(205, 186, 150)
    WHEAT4 = RGB(139, 126, 102)
    WHITE = RGB(255, 255, 255)
    WHITESMOKE = RGB(245, 245, 245)
    WHITESMOKE = RGB(245, 245, 245)
    YELLOW1 = RGB(255, 255, 0)
    YELLOW2 = RGB(238, 238, 0)
    YELLOW3 = RGB(205, 205, 0)
    YELLOW4 = RGB(139, 139, 0)
    """
    Add colors to colors dictionary
    """
    colors['aliceblue'] = ALICEBLUE
    colors['antiquewhite'] = ANTIQUEWHITE
    colors['antiquewhite1'] = ANTIQUEWHITE1
    colors['antiquewhite2'] = ANTIQUEWHITE2
    colors['antiquewhite3'] = ANTIQUEWHITE3
    colors['antiquewhite4'] = ANTIQUEWHITE4
    colors['aqua'] = AQUA
    colors['aquamarine1'] = AQUAMARINE1
    colors['aquamarine2'] = AQUAMARINE2
    colors['aquamarine3'] = AQUAMARINE3
    colors['aquamarine4'] = AQUAMARINE4
    colors['azure1'] = AZURE1
    colors['azure2'] = AZURE2
    colors['azure3'] = AZURE3
    colors['azure4'] = AZURE4
    colors['banana'] = BANANA
    colors['beige'] = BEIGE
    colors['bisque1'] = BISQUE1
    colors['bisque2'] = BISQUE2
    colors['bisque3'] = BISQUE3
    colors['bisque4'] = BISQUE4
    colors['black'] = BLACK
    colors['blanchedalmond'] = BLANCHEDALMOND
    colors['blue'] = BLUE
    colors['blue2'] = BLUE2
    colors['blue3'] = BLUE3
    colors['blue4'] = BLUE4
    colors['blueviolet'] = BLUEVIOLET
    colors['brick'] = BRICK
    colors['brown'] = BROWN
    colors['brown1'] = BROWN1
    colors['brown2'] = BROWN2
    colors['brown3'] = BROWN3
    colors['brown4'] = BROWN4
    colors['burlywood'] = BURLYWOOD
    colors['burlywood1'] = BURLYWOOD1
    colors['burlywood2'] = BURLYWOOD2
    colors['burlywood3'] = BURLYWOOD3
    colors['burlywood4'] = BURLYWOOD4
    colors['burntsienna'] = BURNTSIENNA
    colors['burntumber'] = BURNTUMBER
    colors['cadetblue'] = CADETBLUE
    colors['cadetblue1'] = CADETBLUE1
    colors['cadetblue2'] = CADETBLUE2
    colors['cadetblue3'] = CADETBLUE3
    colors['cadetblue4'] = CADETBLUE4
    colors['cadmiumorange'] = CADMIUMORANGE
    colors['cadmiumyellow'] = CADMIUMYELLOW
    colors['carrot'] = CARROT
    colors['chartreuse1'] = CHARTREUSE1
    colors['chartreuse2'] = CHARTREUSE2
    colors['chartreuse3'] = CHARTREUSE3
    colors['chartreuse4'] = CHARTREUSE4
    colors['chocolate'] = CHOCOLATE
    colors['chocolate1'] = CHOCOLATE1
    colors['chocolate2'] = CHOCOLATE2
    colors['chocolate3'] = CHOCOLATE3
    colors['chocolate4'] = CHOCOLATE4
    colors['cobalt'] = COBALT
    colors['cobaltgreen'] = COBALTGREEN
    colors['coldgrey'] = COLDGREY
    colors['coral'] = CORAL
    colors['coral1'] = CORAL1
    colors['coral2'] = CORAL2
    colors['coral3'] = CORAL3
    colors['coral4'] = CORAL4
    colors['cornflowerblue'] = CORNFLOWERBLUE
    colors['cornsilk1'] = CORNSILK1
    colors['cornsilk2'] = CORNSILK2
    colors['cornsilk3'] = CORNSILK3
    colors['cornsilk4'] = CORNSILK4
    colors['crimson'] = CRIMSON
    colors['cyan2'] = CYAN2
    colors['cyan3'] = CYAN3
    colors['cyan4'] = CYAN4
    colors['darkgoldenrod'] = DARKGOLDENROD
    colors['darkgoldenrod1'] = DARKGOLDENROD1
    colors['darkgoldenrod2'] = DARKGOLDENROD2
    colors['darkgoldenrod3'] = DARKGOLDENROD3
    colors['darkgoldenrod4'] = DARKGOLDENROD4
    colors['darkgray'] = DARKGRAY
    colors['darkgreen'] = DARKGREEN
    colors['darkkhaki'] = DARKKHAKI
    colors['darkolivegreen'] = DARKOLIVEGREEN
    colors['darkolivegreen1'] = DARKOLIVEGREEN1
    colors['darkolivegreen2'] = DARKOLIVEGREEN2
    colors['darkolivegreen3'] = DARKOLIVEGREEN3
    colors['darkolivegreen4'] = DARKOLIVEGREEN4
    colors['darkorange'] = DARKORANGE
    colors['darkorange1'] = DARKORANGE1
    colors['darkorange2'] = DARKORANGE2
    colors['darkorange3'] = DARKORANGE3
    colors['darkorange4'] = DARKORANGE4
    colors['darkorchid'] = DARKORCHID
    colors['darkorchid1'] = DARKORCHID1
    colors['darkorchid2'] = DARKORCHID2
    colors['darkorchid3'] = DARKORCHID3
    colors['darkorchid4'] = DARKORCHID4
    colors['darksalmon'] = DARKSALMON
    colors['darkseagreen'] = DARKSEAGREEN
    colors['darkseagreen1'] = DARKSEAGREEN1
    colors['darkseagreen2'] = DARKSEAGREEN2
    colors['darkseagreen3'] = DARKSEAGREEN3
    colors['darkseagreen4'] = DARKSEAGREEN4
    colors['darkslateblue'] = DARKSLATEBLUE
    colors['darkslategray'] = DARKSLATEGRAY
    colors['darkslategray1'] = DARKSLATEGRAY1
    colors['darkslategray2'] = DARKSLATEGRAY2
    colors['darkslategray3'] = DARKSLATEGRAY3
    colors['darkslategray4'] = DARKSLATEGRAY4
    colors['darkturquoise'] = DARKTURQUOISE
    colors['darkviolet'] = DARKVIOLET
    colors['deeppink1'] = DEEPPINK1
    colors['deeppink2'] = DEEPPINK2
    colors['deeppink3'] = DEEPPINK3
    colors['deeppink4'] = DEEPPINK4
    colors['deepskyblue1'] = DEEPSKYBLUE1
    colors['deepskyblue2'] = DEEPSKYBLUE2
    colors['deepskyblue3'] = DEEPSKYBLUE3
    colors['deepskyblue4'] = DEEPSKYBLUE4
    colors['dimgray'] = DIMGRAY
    colors['dimgray'] = DIMGRAY
    colors['dodgerblue1'] = DODGERBLUE1
    colors['dodgerblue2'] = DODGERBLUE2
    colors['dodgerblue3'] = DODGERBLUE3
    colors['dodgerblue4'] = DODGERBLUE4
    colors['eggshell'] = EGGSHELL
    colors['emeraldgreen'] = EMERALDGREEN
    colors['firebrick'] = FIREBRICK
    colors['firebrick1'] = FIREBRICK1
    colors['firebrick2'] = FIREBRICK2
    colors['firebrick3'] = FIREBRICK3
    colors['firebrick4'] = FIREBRICK4
    colors['flesh'] = FLESH
    colors['floralwhite'] = FLORALWHITE
    colors['forestgreen'] = FORESTGREEN
    colors['gainsboro'] = GAINSBORO
    colors['ghostwhite'] = GHOSTWHITE
    colors['gold1'] = GOLD1
    colors['gold2'] = GOLD2
    colors['gold3'] = GOLD3
    colors['gold4'] = GOLD4
    colors['goldenrod'] = GOLDENROD
    colors['goldenrod1'] = GOLDENROD1
    colors['goldenrod2'] = GOLDENROD2
    colors['goldenrod3'] = GOLDENROD3
    colors['goldenrod4'] = GOLDENROD4
    colors['gray'] = GRAY
    colors['gray1'] = GRAY1
    colors['gray10'] = GRAY10
    colors['gray11'] = GRAY11
    colors['gray12'] = GRAY12
    colors['gray13'] = GRAY13
    colors['gray14'] = GRAY14
    colors['gray15'] = GRAY15
    colors['gray16'] = GRAY16
    colors['gray17'] = GRAY17
    colors['gray18'] = GRAY18
    colors['gray19'] = GRAY19
    colors['gray2'] = GRAY2
    colors['gray20'] = GRAY20
    colors['gray21'] = GRAY21
    colors['gray22'] = GRAY22
    colors['gray23'] = GRAY23
    colors['gray24'] = GRAY24
    colors['gray25'] = GRAY25
    colors['gray26'] = GRAY26
    colors['gray27'] = GRAY27
    colors['gray28'] = GRAY28
    colors['gray29'] = GRAY29
    colors['gray3'] = GRAY3
    colors['gray30'] = GRAY30
    colors['gray31'] = GRAY31
    colors['gray32'] = GRAY32
    colors['gray33'] = GRAY33
    colors['gray34'] = GRAY34
    colors['gray35'] = GRAY35
    colors['gray36'] = GRAY36
    colors['gray37'] = GRAY37
    colors['gray38'] = GRAY38
    colors['gray39'] = GRAY39
    colors['gray4'] = GRAY4
    colors['gray40'] = GRAY40
    colors['gray42'] = GRAY42
    colors['gray43'] = GRAY43
    colors['gray44'] = GRAY44
    colors['gray45'] = GRAY45
    colors['gray46'] = GRAY46
    colors['gray47'] = GRAY47
    colors['gray48'] = GRAY48
    colors['gray49'] = GRAY49
    colors['gray5'] = GRAY5
    colors['gray50'] = GRAY50
    colors['gray51'] = GRAY51
    colors['gray52'] = GRAY52
    colors['gray53'] = GRAY53
    colors['gray54'] = GRAY54
    colors['gray55'] = GRAY55
    colors['gray56'] = GRAY56
    colors['gray57'] = GRAY57
    colors['gray58'] = GRAY58
    colors['gray59'] = GRAY59
    colors['gray6'] = GRAY6
    colors['gray60'] = GRAY60
    colors['gray61'] = GRAY61
    colors['gray62'] = GRAY62
    colors['gray63'] = GRAY63
    colors['gray64'] = GRAY64
    colors['gray65'] = GRAY65
    colors['gray66'] = GRAY66
    colors['gray67'] = GRAY67
    colors['gray68'] = GRAY68
    colors['gray69'] = GRAY69
    colors['gray7'] = GRAY7
    colors['gray70'] = GRAY70
    colors['gray71'] = GRAY71
    colors['gray72'] = GRAY72
    colors['gray73'] = GRAY73
    colors['gray74'] = GRAY74
    colors['gray75'] = GRAY75
    colors['gray76'] = GRAY76
    colors['gray77'] = GRAY77
    colors['gray78'] = GRAY78
    colors['gray79'] = GRAY79
    colors['gray8'] = GRAY8
    colors['gray80'] = GRAY80
    colors['gray81'] = GRAY81
    colors['gray82'] = GRAY82
    colors['gray83'] = GRAY83
    colors['gray84'] = GRAY84
    colors['gray85'] = GRAY85
    colors['gray86'] = GRAY86
    colors['gray87'] = GRAY87
    colors['gray88'] = GRAY88
    colors['gray89'] = GRAY89
    colors['gray9'] = GRAY9
    colors['gray90'] = GRAY90
    colors['gray91'] = GRAY91
    colors['gray92'] = GRAY92
    colors['gray93'] = GRAY93
    colors['gray94'] = GRAY94
    colors['gray95'] = GRAY95
    colors['gray97'] = GRAY97
    colors['gray98'] = GRAY98
    colors['gray99'] = GRAY99
    colors['green'] = GREEN
    colors['green1'] = GREEN1
    colors['green2'] = GREEN2
    colors['green3'] = GREEN3
    colors['green4'] = GREEN4
    colors['greenyellow'] = GREENYELLOW
    colors['honeydew1'] = HONEYDEW1
    colors['honeydew2'] = HONEYDEW2
    colors['honeydew3'] = HONEYDEW3
    colors['honeydew4'] = HONEYDEW4
    colors['hotpink'] = HOTPINK
    colors['hotpink1'] = HOTPINK1
    colors['hotpink2'] = HOTPINK2
    colors['hotpink3'] = HOTPINK3
    colors['hotpink4'] = HOTPINK4
    colors['indianred'] = INDIANRED
    colors['indianred'] = INDIANRED
    colors['indianred1'] = INDIANRED1
    colors['indianred2'] = INDIANRED2
    colors['indianred3'] = INDIANRED3
    colors['indianred4'] = INDIANRED4
    colors['indigo'] = INDIGO
    colors['ivory1'] = IVORY1
    colors['ivory2'] = IVORY2
    colors['ivory3'] = IVORY3
    colors['ivory4'] = IVORY4
    colors['ivoryblack'] = IVORYBLACK
    colors['khaki'] = KHAKI
    colors['khaki1'] = KHAKI1
    colors['khaki2'] = KHAKI2
    colors['khaki3'] = KHAKI3
    colors['khaki4'] = KHAKI4
    colors['lavender'] = LAVENDER
    colors['lavenderblush1'] = LAVENDERBLUSH1
    colors['lavenderblush2'] = LAVENDERBLUSH2
    colors['lavenderblush3'] = LAVENDERBLUSH3
    colors['lavenderblush4'] = LAVENDERBLUSH4
    colors['lawngreen'] = LAWNGREEN
    colors['lemonchiffon1'] = LEMONCHIFFON1
    colors['lemonchiffon2'] = LEMONCHIFFON2
    colors['lemonchiffon3'] = LEMONCHIFFON3
    colors['lemonchiffon4'] = LEMONCHIFFON4
    colors['lightblue'] = LIGHTBLUE
    colors['lightblue1'] = LIGHTBLUE1
    colors['lightblue2'] = LIGHTBLUE2
    colors['lightblue3'] = LIGHTBLUE3
    colors['lightblue4'] = LIGHTBLUE4
    colors['lightcoral'] = LIGHTCORAL
    colors['lightcyan1'] = LIGHTCYAN1
    colors['lightcyan2'] = LIGHTCYAN2
    colors['lightcyan3'] = LIGHTCYAN3
    colors['lightcyan4'] = LIGHTCYAN4
    colors['lightgoldenrod1'] = LIGHTGOLDENROD1
    colors['lightgoldenrod2'] = LIGHTGOLDENROD2
    colors['lightgoldenrod3'] = LIGHTGOLDENROD3
    colors['lightgoldenrod4'] = LIGHTGOLDENROD4
    colors['lightgoldenrodyellow'] = LIGHTGOLDENRODYELLOW
    colors['lightgrey'] = LIGHTGREY
    colors['lightpink'] = LIGHTPINK
    colors['lightpink1'] = LIGHTPINK1
    colors['lightpink2'] = LIGHTPINK2
    colors['lightpink3'] = LIGHTPINK3
    colors['lightpink4'] = LIGHTPINK4
    colors['lightsalmon1'] = LIGHTSALMON1
    colors['lightsalmon2'] = LIGHTSALMON2
    colors['lightsalmon3'] = LIGHTSALMON3
    colors['lightsalmon4'] = LIGHTSALMON4
    colors['lightseagreen'] = LIGHTSEAGREEN
    colors['lightskyblue'] = LIGHTSKYBLUE
    colors['lightskyblue1'] = LIGHTSKYBLUE1
    colors['lightskyblue2'] = LIGHTSKYBLUE2
    colors['lightskyblue3'] = LIGHTSKYBLUE3
    colors['lightskyblue4'] = LIGHTSKYBLUE4
    colors['lightslateblue'] = LIGHTSLATEBLUE
    colors['lightslategray'] = LIGHTSLATEGRAY
    colors['lightsteelblue'] = LIGHTSTEELBLUE
    colors['lightsteelblue1'] = LIGHTSTEELBLUE1
    colors['lightsteelblue2'] = LIGHTSTEELBLUE2
    colors['lightsteelblue3'] = LIGHTSTEELBLUE3
    colors['lightsteelblue4'] = LIGHTSTEELBLUE4
    colors['lightyellow1'] = LIGHTYELLOW1
    colors['lightyellow2'] = LIGHTYELLOW2
    colors['lightyellow3'] = LIGHTYELLOW3
    colors['lightyellow4'] = LIGHTYELLOW4
    colors['limegreen'] = LIMEGREEN
    colors['linen'] = LINEN
    colors['magenta'] = MAGENTA
    colors['magenta2'] = MAGENTA2
    colors['magenta3'] = MAGENTA3
    colors['magenta4'] = MAGENTA4
    colors['manganeseblue'] = MANGANESEBLUE
    colors['maroon'] = MAROON
    colors['maroon1'] = MAROON1
    colors['maroon2'] = MAROON2
    colors['maroon3'] = MAROON3
    colors['maroon4'] = MAROON4
    colors['mediumorchid'] = MEDIUMORCHID
    colors['mediumorchid1'] = MEDIUMORCHID1
    colors['mediumorchid2'] = MEDIUMORCHID2
    colors['mediumorchid3'] = MEDIUMORCHID3
    colors['mediumorchid4'] = MEDIUMORCHID4
    colors['mediumpurple'] = MEDIUMPURPLE
    colors['mediumpurple1'] = MEDIUMPURPLE1
    colors['mediumpurple2'] = MEDIUMPURPLE2
    colors['mediumpurple3'] = MEDIUMPURPLE3
    colors['mediumpurple4'] = MEDIUMPURPLE4
    colors['mediumseagreen'] = MEDIUMSEAGREEN
    colors['mediumslateblue'] = MEDIUMSLATEBLUE
    colors['mediumspringgreen'] = MEDIUMSPRINGGREEN
    colors['mediumturquoise'] = MEDIUMTURQUOISE
    colors['mediumvioletred'] = MEDIUMVIOLETRED
    colors['melon'] = MELON
    colors['midnightblue'] = MIDNIGHTBLUE
    colors['mint'] = MINT
    colors['mintcream'] = MINTCREAM
    colors['mistyrose1'] = MISTYROSE1
    colors['mistyrose2'] = MISTYROSE2
    colors['mistyrose3'] = MISTYROSE3
    colors['mistyrose4'] = MISTYROSE4
    colors['moccasin'] = MOCCASIN
    colors['navajowhite1'] = NAVAJOWHITE1
    colors['navajowhite2'] = NAVAJOWHITE2
    colors['navajowhite3'] = NAVAJOWHITE3
    colors['navajowhite4'] = NAVAJOWHITE4
    colors['navy'] = NAVY
    colors['oldlace'] = OLDLACE
    colors['olive'] = OLIVE
    colors['olivedrab'] = OLIVEDRAB
    colors['olivedrab1'] = OLIVEDRAB1
    colors['olivedrab2'] = OLIVEDRAB2
    colors['olivedrab3'] = OLIVEDRAB3
    colors['olivedrab4'] = OLIVEDRAB4
    colors['orange'] = ORANGE
    colors['orange1'] = ORANGE1
    colors['orange2'] = ORANGE2
    colors['orange3'] = ORANGE3
    colors['orange4'] = ORANGE4
    colors['orangered1'] = ORANGERED1
    colors['orangered2'] = ORANGERED2
    colors['orangered3'] = ORANGERED3
    colors['orangered4'] = ORANGERED4
    colors['orchid'] = ORCHID
    colors['orchid1'] = ORCHID1
    colors['orchid2'] = ORCHID2
    colors['orchid3'] = ORCHID3
    colors['orchid4'] = ORCHID4
    colors['palegoldenrod'] = PALEGOLDENROD
    colors['palegreen'] = PALEGREEN
    colors['palegreen1'] = PALEGREEN1
    colors['palegreen2'] = PALEGREEN2
    colors['palegreen3'] = PALEGREEN3
    colors['palegreen4'] = PALEGREEN4
    colors['paleturquoise1'] = PALETURQUOISE1
    colors['paleturquoise2'] = PALETURQUOISE2
    colors['paleturquoise3'] = PALETURQUOISE3
    colors['paleturquoise4'] = PALETURQUOISE4
    colors['palevioletred'] = PALEVIOLETRED
    colors['palevioletred1'] = PALEVIOLETRED1
    colors['palevioletred2'] = PALEVIOLETRED2
    colors['palevioletred3'] = PALEVIOLETRED3
    colors['palevioletred4'] = PALEVIOLETRED4
    colors['papayawhip'] = PAPAYAWHIP
    colors['peachpuff1'] = PEACHPUFF1
    colors['peachpuff2'] = PEACHPUFF2
    colors['peachpuff3'] = PEACHPUFF3
    colors['peachpuff4'] = PEACHPUFF4
    colors['peacock'] = PEACOCK
    colors['pink'] = PINK
    colors['pink1'] = PINK1
    colors['pink2'] = PINK2
    colors['pink3'] = PINK3
    colors['pink4'] = PINK4
    colors['plum'] = PLUM
    colors['plum1'] = PLUM1
    colors['plum2'] = PLUM2
    colors['plum3'] = PLUM3
    colors['plum4'] = PLUM4
    colors['powderblue'] = POWDERBLUE
    colors['purple'] = PURPLE
    colors['purple1'] = PURPLE1
    colors['purple2'] = PURPLE2
    colors['purple3'] = PURPLE3
    colors['purple4'] = PURPLE4
    colors['raspberry'] = RASPBERRY
    colors['rawsienna'] = RAWSIENNA
    colors['red'] = RED
    colors['red1'] = RED1
    colors['red2'] = RED2
    colors['red3'] = RED3
    colors['red4'] = RED4
    colors['rosybrown'] = ROSYBROWN
    colors['rosybrown1'] = ROSYBROWN1
    colors['rosybrown2'] = ROSYBROWN2
    colors['rosybrown3'] = ROSYBROWN3
    colors['rosybrown4'] = ROSYBROWN4
    colors['royalblue'] = ROYALBLUE
    colors['royalblue1'] = ROYALBLUE1
    colors['royalblue2'] = ROYALBLUE2
    colors['royalblue3'] = ROYALBLUE3
    colors['royalblue4'] = ROYALBLUE4
    colors['salmon'] = SALMON
    colors['salmon1'] = SALMON1
    colors['salmon2'] = SALMON2
    colors['salmon3'] = SALMON3
    colors['salmon4'] = SALMON4
    colors['sandybrown'] = SANDYBROWN
    colors['sapgreen'] = SAPGREEN
    colors['seagreen1'] = SEAGREEN1
    colors['seagreen2'] = SEAGREEN2
    colors['seagreen3'] = SEAGREEN3
    colors['seagreen4'] = SEAGREEN4
    colors['seashell1'] = SEASHELL1
    colors['seashell2'] = SEASHELL2
    colors['seashell3'] = SEASHELL3
    colors['seashell4'] = SEASHELL4
    colors['sepia'] = SEPIA
    colors['sgibeet'] = SGIBEET
    colors['sgibrightgray'] = SGIBRIGHTGRAY
    colors['sgichartreuse'] = SGICHARTREUSE
    colors['sgidarkgray'] = SGIDARKGRAY
    colors['sgigray12'] = SGIGRAY12
    colors['sgigray16'] = SGIGRAY16
    colors['sgigray32'] = SGIGRAY32
    colors['sgigray36'] = SGIGRAY36
    colors['sgigray52'] = SGIGRAY52
    colors['sgigray56'] = SGIGRAY56
    colors['sgigray72'] = SGIGRAY72
    colors['sgigray76'] = SGIGRAY76
    colors['sgigray92'] = SGIGRAY92
    colors['sgigray96'] = SGIGRAY96
    colors['sgilightblue'] = SGILIGHTBLUE
    colors['sgilightgray'] = SGILIGHTGRAY
    colors['sgiolivedrab'] = SGIOLIVEDRAB
    colors['sgisalmon'] = SGISALMON
    colors['sgislateblue'] = SGISLATEBLUE
    colors['sgiteal'] = SGITEAL
    colors['sienna'] = SIENNA
    colors['sienna1'] = SIENNA1
    colors['sienna2'] = SIENNA2
    colors['sienna3'] = SIENNA3
    colors['sienna4'] = SIENNA4
    colors['silver'] = SILVER
    colors['skyblue'] = SKYBLUE
    colors['skyblue1'] = SKYBLUE1
    colors['skyblue2'] = SKYBLUE2
    colors['skyblue3'] = SKYBLUE3
    colors['skyblue4'] = SKYBLUE4
    colors['slateblue'] = SLATEBLUE
    colors['slateblue1'] = SLATEBLUE1
    colors['slateblue2'] = SLATEBLUE2
    colors['slateblue3'] = SLATEBLUE3
    colors['slateblue4'] = SLATEBLUE4
    colors['slategray'] = SLATEGRAY
    colors['slategray1'] = SLATEGRAY1
    colors['slategray2'] = SLATEGRAY2
    colors['slategray3'] = SLATEGRAY3
    colors['slategray4'] = SLATEGRAY4
    colors['snow1'] = SNOW1
    colors['snow2'] = SNOW2
    colors['snow3'] = SNOW3
    colors['snow4'] = SNOW4
    colors['springgreen'] = SPRINGGREEN
    colors['springgreen1'] = SPRINGGREEN1
    colors['springgreen2'] = SPRINGGREEN2
    colors['springgreen3'] = SPRINGGREEN3
    colors['steelblue'] = STEELBLUE
    colors['steelblue1'] = STEELBLUE1
    colors['steelblue2'] = STEELBLUE2
    colors['steelblue3'] = STEELBLUE3
    colors['steelblue4'] = STEELBLUE4
    colors['tan'] = TAN
    colors['tan1'] = TAN1
    colors['tan2'] = TAN2
    colors['tan3'] = TAN3
    colors['tan4'] = TAN4
    colors['teal'] = TEAL
    colors['thistle'] = THISTLE
    colors['thistle1'] = THISTLE1
    colors['thistle2'] = THISTLE2
    colors['thistle3'] = THISTLE3
    colors['thistle4'] = THISTLE4
    colors['tomato1'] = TOMATO1
    colors['tomato2'] = TOMATO2
    colors['tomato3'] = TOMATO3
    colors['tomato4'] = TOMATO4
    colors['turquoise'] = TURQUOISE
    colors['turquoise1'] = TURQUOISE1
    colors['turquoise2'] = TURQUOISE2
    colors['turquoise3'] = TURQUOISE3
    colors['turquoise4'] = TURQUOISE4
    colors['turquoiseblue'] = TURQUOISEBLUE
    colors['violet'] = VIOLET
    colors['violetred'] = VIOLETRED
    colors['violetred1'] = VIOLETRED1
    colors['violetred2'] = VIOLETRED2
    colors['violetred3'] = VIOLETRED3
    colors['violetred4'] = VIOLETRED4
    colors['warmgrey'] = WARMGREY
    colors['wheat'] = WHEAT
    colors['wheat1'] = WHEAT1
    colors['wheat2'] = WHEAT2
    colors['wheat3'] = WHEAT3
    colors['wheat4'] = WHEAT4
    colors['white'] = WHITE
    colors['whitesmoke'] = WHITESMOKE
    colors['whitesmoke'] = WHITESMOKE
    colors['yellow1'] = YELLOW1
    colors['yellow2'] = YELLOW2
    colors['yellow3'] = YELLOW3
    colors['yellow4'] = YELLOW4
    """
    Ordered Dict
    """
    colors = OrderedDict(sorted(colors.items(), key=lambda t: t[0]))



# import pandas as pd
from enum import Enum


class PhConstants:
    # Mutable Data Type; use copy.copy(x) to create object without reference
    DICT_EMPTY = {}
    # Mutable Data Type; use copy.copy(x) to create object without reference
    LIST_EMPTY = []
    # Mutable Data Type; use copy.copy(x) to create object without reference
    SET_EMPTY = set()
    # Immutable Data Type
    TUPLE_EMPTY = ((),)
    # Immutable Data Type
    STR_EMPTY = ''
    # TODO: Causing issues in setup.py as module is not installed during setup.
    # No Module Names Pandas
    # Value Mutable; but favor immutability
    # DF_EMPTY = pd.DataFrame()

    STR_TAB = '    '
    STR_SPACE = ' '

    STR_CURLY_BRACE_START = '{'
    STR_CURLY_BRACE_END = '}'

    OFFSET_ZERO = 0
    OFFSET_ONE = 1
    OFFSET_NAME_FOR_OUTPUT_FILE = 0
    OFFSET_NAME_FOR_ASN_FILE = 1
    KEYWORD_VARIABLE_DECLARATION_IN = ['var_In', 'var_in_list', 'var_in']
    KEYWORD_VARIABLE_DECLARATION_OUT = ['var_Out', 'varlist', 'var_out', 'VAR_OUT', 'Var_Out']
    KEYWORD_VARIABLE_QUANTITY = ['Quantity', 'Count']
    PLACE_HOLDERS_FOR_VARIABLES = ['${VAR}', '${VAR  }', '${VAR    }']

    # 0th offset contains most preferred name for output file
    # 1st offset contains most preferred name for ASN file (if Applicable)
    VAR_POOL_ICCID_WO_CHECKSUM = ['SER_NB', 'Ser_nb']
    VAR_POOL_ICCID = ['ICCID', 'iccid', 'ICCID_FROM'] + VAR_POOL_ICCID_WO_CHECKSUM
    VAR_POOL_IMSI = ['IMSI', 'imsi', 'IMSI_FROM']
    VAR_POOL_IMSI2 = ['imsi2', 'imsi-9009-2601-rec2']
    VAR_POOL_KI = ['KI', 'key', 'ki', 'KEY']
    VAR_POOL_PIN1 = ['PIN1', 'pinAppl1', 'CHV1', 'PIN', 'CHV']
    VAR_POOL_PIN2 = ['PIN2', 'pinAppl2', 'CHV2', ]
    VAR_POOL_PUK1 = ['PUK1', 'pukAppl1', 'PUK']
    VAR_POOL_PUK2 = ['PUK2', 'pukAppl2']
    VAR_POOL_ADM1 = ['ADM1', 'adm1', 'ISC1', 'ADM1_0A', 'ADM', 'ADMIN']
    VAR_POOL_ADM2 = ['ADM2', 'adm2', 'ISC2', 'ADM1_0B']
    VAR_POOL_ADM3 = ['ADM3', 'adm3', 'ISC3', 'ADM1_0C']
    VAR_POOL_ADM4 = ['ADM4', 'adm4', 'ISC4', 'ADM1_0D']
    VAR_POOL_ADM5 = ['ADM5', 'adm5', 'ISC5', 'ADM1_0E']
    VAR_POOL_KIC1 = ['KIC1', 'kic1IsdR', 'KIC', 'KIC_KID']
    VAR_POOL_KID1 = ['KID1', 'kid1IsdR', 'KID', 'KIC_KID']
    VAR_POOL_KIK1 = ['KIK1', 'kik1IsdR', 'KIK']
    VAR_POOL_KIC2 = ['KIC2', 'kic1IsdP']
    VAR_POOL_KID2 = ['KID2', 'kid1IsdP']
    VAR_POOL_KIK2 = ['KIK2', 'kik1IsdP']
    VAR_POOL_MNO_SD_KIC_01 = ['KIC01', 'mno_sd_kic_01']
    VAR_POOL_MNO_SD_KID_01 = ['KID01', 'mno_sd_kid_01']
    VAR_POOL_MNO_SD_KIK_01 = ['KIK01', 'mno_sd_kik_01']
    VAR_POOL_MNO_SD_KIC_02 = ['KIC02', 'mno_sd_kic_02']
    VAR_POOL_MNO_SD_KID_02 = ['KID02', 'mno_sd_kid_02']
    VAR_POOL_MNO_SD_KIK_02 = ['KIK02', 'mno_sd_kik_02']
    VAR_POOL_MNO_SD_KIC_03 = ['KIC03', 'mno_sd_kic_03']
    VAR_POOL_MNO_SD_KID_03 = ['KID03', 'mno_sd_kid_03']
    VAR_POOL_MNO_SD_KIK_03 = ['KIK03', 'mno_sd_kik_03']
    VAR_POOL_MNO_SD_KIC_04 = ['KIC04', 'mno_sd_kic_04']
    VAR_POOL_MNO_SD_KID_04 = ['KID04', 'mno_sd_kid_04']
    VAR_POOL_MNO_SD_KIK_04 = ['KIK04', 'mno_sd_kik_04']
    VAR_POOL_MNO_SD_KIC_05 = ['KIC05', 'mno_sd_kic_05']
    VAR_POOL_MNO_SD_KID_05 = ['KID05', 'mno_sd_kid_05']
    VAR_POOL_MNO_SD_KIK_05 = ['KIK05', 'mno_sd_kik_05']
    VAR_POOL_MNO_SD_KIC_06 = ['KIC06', 'mno_sd_kic_06']
    VAR_POOL_MNO_SD_KID_06 = ['KID06', 'mno_sd_kid_06']
    VAR_POOL_MNO_SD_KIK_06 = ['KIK06', 'mno_sd_kik_06']
    VAR_POOL_MNO_SD_KIC_07 = ['KIC07', 'mno_sd_kic_07']
    VAR_POOL_MNO_SD_KID_07 = ['KID07', 'mno_sd_kid_07']
    VAR_POOL_MNO_SD_KIK_07 = ['KIK07', 'mno_sd_kik_07']
    VAR_POOL_MNO_SD_KIC_08 = ['KIC08', 'mno_sd_kic_08']
    VAR_POOL_MNO_SD_KID_08 = ['KID08', 'mno_sd_kid_08']
    VAR_POOL_MNO_SD_KIK_08 = ['KIK08', 'mno_sd_kik_08']
    VAR_POOL_MNO_SD_KIC_09 = ['KIC09', 'mno_sd_kic_09']
    VAR_POOL_MNO_SD_KID_09 = ['KID09', 'mno_sd_kid_09']
    VAR_POOL_MNO_SD_KIK_09 = ['KIK09', 'mno_sd_kik_09']
    VAR_POOL_MNO_SD_KIC_0A = ['KIC0A', 'mno_sd_kic_0A']
    VAR_POOL_MNO_SD_KID_0A = ['KID0A', 'mno_sd_kid_0A']
    VAR_POOL_MNO_SD_KIK_0A = ['KIK0A', 'mno_sd_kik_0A']
    VAR_POOL_MNO_SD_KIC_0B = ['KIC0B', 'mno_sd_kic_0B']
    VAR_POOL_MNO_SD_KID_0B = ['KID0B', 'mno_sd_kid_0B']
    VAR_POOL_MNO_SD_KIK_0B = ['KIK0B', 'mno_sd_kik_0B']
    VAR_POOL_MNO_SD_KIC_0C = ['KIC0C', 'mno_sd_kic_0C']
    VAR_POOL_MNO_SD_KID_0C = ['KID0C', 'mno_sd_kid_0C']
    VAR_POOL_MNO_SD_KIK_0C = ['KIK0C', 'mno_sd_kik_0C']
    VAR_POOL_MNO_SD_KIC_0D = ['KIC0D', 'mno_sd_kic_0D']
    VAR_POOL_MNO_SD_KID_0D = ['KID0D', 'mno_sd_kid_0D']
    VAR_POOL_MNO_SD_KIK_0D = ['KIK0D', 'mno_sd_kik_0D']
    VAR_POOL_MNO_SD_KIC_0E = ['KIC0E', 'mno_sd_kic_0E']
    VAR_POOL_MNO_SD_KID_0E = ['KID0E', 'mno_sd_kid_0E']
    VAR_POOL_MNO_SD_KIK_0E = ['KIK0E', 'mno_sd_kik_0E']
    VAR_POOL_MNO_SD_KIC_0F = ['KIC0F', 'mno_sd_kic_0F']
    VAR_POOL_MNO_SD_KID_0F = ['KID0F', 'mno_sd_kid_0F']
    VAR_POOL_MNO_SD_KIK_0F = ['KIK0F', 'mno_sd_kik_0F']
    VAR_POOL_2_PIN1 = ['2PIN1', 'secondPINAppl1']
    VAR_POOL_2_PUK1 = ['2PUK1', 'secondPUKAppl1']
    VAR_POOL_3_PIN1 = ['3PIN1', 'secondPINAppl2']
    VAR_POOL_3_PUK1 = ['3PUK1', 'secondPUKAppl2']
    VAR_POOL_ACC = ['ACC', 'ef-acc', 'Access_Control']

    VAR_TYPE_IN = 1
    VAR_TYPE_OUT = 2
    VAR_TYPE_PROFILE = 3

    POOL_4_DIGITS_LENGTH = [
        'PIN1', 'PIN2', 'secondPINAppl1', 'secondPINAppl2'
    ]
    POOL_8_DIGITS_LENGTH = [
        'PUK1', 'PUK2', 'ADM1', 'ADM2', 'ADM3', 'ADM4', 'ADM5', 'secondPUKAppl1', 'secondPUKAppl2'
    ]
    POOL_16_BYTES_LENGTH = [
        'KI',
        'KIC1', 'KID1', 'KIK1',
        'KIC2', 'KID2', 'KIK2',
        'KIC01', 'KID01', 'KIK01',
        'KIC02', 'KID02', 'KIK02',
        'KIC03', 'KID03', 'KIK03',
        'KIC04', 'KID04', 'KIK04',
        'KIC05', 'KID05', 'KIK05',
        'KIC06', 'KID06', 'KIK06',
        'KIC07', 'KID07', 'KIK07',
        'KIC08', 'KID08', 'KIK08',
        'KIC09', 'KID09', 'KIK09',
        'KIC0A', 'KID0A', 'KIK0A',
        'KIC0B', 'KID0B', 'KIK0B',
        'KIC0C', 'KID0C', 'KIK0C',
        'KIC0D', 'KID0D', 'KIK0D',
        'KIC0E', 'KID0E', 'KIK0E',
        'KIC0F', 'KID0F', 'KIK0F',
    ]

    STR_TYPE_NUMERIC = 1
    STR_TYPE_NUMERIC_OCT = 2
    STR_TYPE_NUMERIC_BIN = 3

    STR_TYPE_HEX_LOWER_CASE = 11
    STR_TYPE_HEX_UPPER_CASE = 12
    STR_TYPE_HEX_RANDOM_CASE = 13

    STR_TYPE_ASCII_LOWER_CASE = 21
    STR_TYPE_ASCII_UPPER_CASE = 22
    STR_TYPE_ASCII_RANDOM_CASE = 32

    STR_TYPE_ALPHA_NUMERIC_LOWER_CASE = 30
    STR_TYPE_ALPHA_NUMERIC_UPPER_CASE = 31
    STR_TYPE_ALPHA_NUMERIC_RANDOM_CASE = 32

    STR_TYPE_PASSWORD_LOWER_CASE = 40
    STR_TYPE_PASSWORD_UPPER_CASE = 41
    STR_TYPE_PASSWORD_RANDOM_CASE = 42

    FILE_TYPE_IN = 1
    FILE_TYPE_OUT = 2

    AID_EUICC_ISDR_OR_UICC_ISD = 'A000000151000000'
    AID_EUICC_ISDP_INITIAL = 'A0000005591010'

    LENGTH_EF_ACC = 0x02
    LENGTH_IMSI = 0x08
    LENGTH_EF_IMSI = LENGTH_IMSI + 0x01
    LENGTH_PINS_PUKS_ADMS = 0x08
    LENGTH_SPN_STRING = 0x10
    LENGTH_EF_SPN = LENGTH_SPN_STRING + 0x01
    LENGTH_DIALING_NUMBER_DEFAULT = 0x0C
    LENGTH_SMSC = LENGTH_DIALING_NUMBER_DEFAULT
    LENGTH_MSISDN = LENGTH_DIALING_NUMBER_DEFAULT

    ONE_STEP_FORWARD = '/'
    MULTIPLE_STEPS_FORWARD = '/**/'
    ONE_STEP_BACKWARD = '/../'

    TRUE_VALUE_POOL = ('yes', 'true', 't', 'y', '1')
    FALSE_VALUE_POOL = ('no', 'false', 'f', 'n', '0')
    EXIT_VALUE_POOL = ('e', 'exit')
    USER_INPUT_OPTIONS = ' [Yes/No/Exit]: '
    MAX_SUPPORTED_DIGIT_IN_INT = 9
    FORMAT_HEX_STRING_AS_PACK = 1
    FORMAT_HEX_STRING_AS_HEX = 2
    FORMAT_HEX_STRING_AS_UPPERCASE = 4
    FORMAT_HEX_STRING_AS_COMMA = 8

    SEARCH_TYPE_PLAIN = 1
    SEARCH_TYPE_REGEX = 2

    FILE_EXTN_TMP = '.tmp'

    SEPERATOR_TWO_WORDS = ' '
    SEPERATOR_TWO_LINES = '\n'
    SEPERATOR_TWO_DATA_SET = '     -----     '
    SEPERATOR_TWO_LINES_MULTI = '\n\n'
    SEPERATOR_FILE_NAME = '_'
    SEPERATOR_ONE_LINE = ': '
    SEPERATOR_KEY_VALUE = ': '
    SEPERATOR_INFO = ' => '
    SEPERATOR_MULTI_LINE = ':\n'
    SEPERATOR_MULTI_LINE_TABBED = ':\n\t'
    SEPERATOR_MULTI_OBJ = '; '
    SEPERATOR_NAME_VERSION = ' '
    SEPERATOR_TITLE = ' | '

    ENCLOSE_HEX = 1
    ENCLOSE_NAME_VALUE = 2
    ENCLOSE_NAME_VALUE_HEX = 3
    ENCLOSE_NAME_VALUE_DICT = 4
    ENCLOSE_NAME_VALUE_HEX_DICT = 5
    ENCLOSE_NAME_VALUE_SEQ_DICT = 6
    ENCLOSE_NAME_VALUE_SEQ = 7
    ENCLOSE_COMMENT = 8

    DEFAULT_TRIM_STRING = '...'
    DEFAULT_TRIM_STRING_LENGTH = len(DEFAULT_TRIM_STRING)

    HEADING_LENGTH_MAX = 120
    HEADING_LENGTH_RESERVE_STARTING_AND_ENDING_SYMBOL = 2
    HEADING_LENGTH_RESERVE_STARTING_AND_ENDING_WHITE_SPACES = 2

    REMARKS_MAX_LENGTH = HEADING_LENGTH_MAX - HEADING_LENGTH_RESERVE_STARTING_AND_ENDING_SYMBOL - HEADING_LENGTH_RESERVE_STARTING_AND_ENDING_WHITE_SPACES

    START_BLOCK = 1
    END_BLOCK = 2
    KNOWN = 'Known'
    UNKNOWN = 'UnKnown'
    PARENT = 21
    CHILD = 22
    DETAILS = 'Details'
    SUMMARY = 'Summary'
    EXCEPTION_OCCURRED = 'Exception Occurred'
    EXCEPTION_OCCURRED_AT_FUNC = 'Exception Occurred at function'
    EXPORT_ERROR = 'export error'
    READ_WRITE_ERROR = 'input/output path reading/writing error'
    WRITE_PATH_ERROR = 'output path writing error'
    TIME_OUT_ERROR = 'time out error'
    PACKAGE_NOT_FOUND_ERROR = 'package not found error'
    NON_ZERO_EXIT_STATUS_ERROR = 'non-zero exit status error'
    INPUTS_ERROR = 'Check all your Inputs'
    INVALID_RAW_DATA = 'Check your Raw Data'
    INVALID_INPUT_DATA = 'Check your Input Data'
    MISSING_INPUT_DATA = 'Missing Input Data'
    MISSING_INPUT_YML = 'Missing "input" in Yml Config'
    INCORRECT_BLOCK_STRUCTURE = 'Incorrect Block Structure; Opening and Closing Sequences are not matching'
    EMPTY_INPUT_FILE = 'Empty Input File'
    UNKNOWN_INPUT_FORMAT = 'Unknown Input Format'
    INVALID_TIME_OUT = 'Invalid Time Out'
    INVALID_URL_TIME_OUT = 'Invalid URL Time Out'
    ITEM = 'item'
    SUB = 'sub '
    KEY_DATE_TIME = 'Date & Time'

    ENV_VARIABLES = {}

    PUBLIC_IP_SERVICES_POOL = [
        'https://ifconfig.co/ip',  # Seems machine based protocol (IPV4/IPV6)
        'https://ipecho.net/plain',  # Seems machine based protocol (IPV4/IPV6)
        'https://checkip.amazonaws.com',  # Seems only IPV4
        'https://ipinfo.io/ip',  # Dedicated IPV4 & IPV6
        'https://api.ipify.org',  # Dedicated IPV4 & IPV6
        'https://icanhazip.com/',  # Dedicated IPV4 & IPV6
        'https://ident.me',  # Dedicated IPV4 & IPV6
    ]

    PUBLIC_IP_SERVICES_IPV4_POOL = [
        'https://ipinfo.io/ip',
        'https://api.ipify.org/',
        'https://ipv4.icanhazip.com/',
        'https://v4.ident.me/',
    ]
    PUBLIC_IP_SERVICES_IPV6_POOL = [
        'https://v6.ipinfo.io/ip',
        'https://api64.ipify.org/',
        'https://ipv6.icanhazip.com/',
        'https://v6.ident.me/',
    ]

    STR_SELECT_OPTION = '-- Select --'
    STR_OTHER_OPTION = '-- Other --'

    CHAR_ENCODING_ASCII = 'ascii'
    CHAR_ENCODING_UTF8 = 'utf-8'
    CHAR_ENCODING_ISO_8859 = 'ISO-8859-1'
    CHAR_ENCODING_CP_1252 = 'cp1252'

    CHAR_ENCODING_POOL = [
        CHAR_ENCODING_ASCII,
        CHAR_ENCODING_UTF8,
        CHAR_ENCODING_ISO_8859,
        CHAR_ENCODING_CP_1252,
    ]

    CHAR_ENCODING_ERRORS_STRICT = 'strict'
    CHAR_ENCODING_ERRORS_IGNORE = 'ignore'
    CHAR_ENCODING_ERRORS_REPLACE = 'replace'

    # Copied from "venv/Lib/site-packages/pandas/_typing.py"
    CHAR_ENCODING_ERRORS_POOL = [
        CHAR_ENCODING_ERRORS_STRICT,
        CHAR_ENCODING_ERRORS_IGNORE,
        CHAR_ENCODING_ERRORS_REPLACE,
        'surrogateescape',
        'xmlcharrefreplace',
        'backslashreplace',
        'namereplace',
    ]

    CRLF = '\r\n'
    LF = '\n'
    WINDOWS_LINE_ENDING = CRLF
    UNIX_LINE_ENDING = LF

    BEGIN_CERTIFICATE = '-----BEGIN CERTIFICATE-----'
    END_CERTIFICATE = '-----END CERTIFICATE-----'
    CERTIFICATE = 'CERTIFICATE'
    OPEN_SSL_CERTIFICATE_COMMENTS = '-'

    YES = True
    NO = False

    DIR_CREATION = 1
    DIR_DELETION = 2

    TEST_DATA_A_Z_LOWER_LEN_26 = 'abcdefghijklmnopqrstuvwxyz'
    TEST_DATA_A_Z_UPPER_LEN_26 = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ'
    TEST_DATA_0_9_LEN_10 = '0123456789'
    TEST_DATA_SYMBOL_LEN_13 = '~!@#$%^&*()_+'
    TEST_DATA_MIX_LEN_75 = TEST_DATA_A_Z_LOWER_LEN_26 + TEST_DATA_0_9_LEN_10 + TEST_DATA_A_Z_UPPER_LEN_26 + TEST_DATA_SYMBOL_LEN_13

    # '**DEPRECATED**'
    # DECODE_MODE_ASCII = 'ascii'
    # '**DEPRECATED**'
    # DECODE_MODE_UTF8 = 'utf-8'
    # '**DEPRECATED**'
    # STR_ENCODING_FORMAT_ASCII = 'ascii'
    # '**DEPRECATED**'
    # STR_ENCODING_FORMAT_UTF8 = 'utf-8'
    # '**DEPRECATED**'
    # STR_ENCODING_FORMAT_ISO_8859 = 'ISO-8859-1'
    # '**DEPRECATED**'
    # STR_ENCODING_FORMAT_CP_1252 = 'cp1252'
    # '**DEPRECATED**'
    # STR_ENCODING_ERRORS_IGNORE = 'ignore'
    # '**DEPRECATED**'
    # STR_ENCODING_ERRORS_REPLACE = 'replace'
    # '**DEPRECATED**'
    # STR_TYPE_HEX = STR_TYPE_HEX_UPPER_CASE
    # '**DEPRECATED**'
    # STR_TYPE_PLAIN = STR_TYPE_ALPHA_NUMERIC_RANDOM_CASE

    class Position(Enum):
        TOP = 1
        RIGHT = 2
        BOTTOM = 3
        LEFT = 4



import re

from play_helpers.ph_constants import PhConstants
from play_helpers.ph_keys import PhKeys


class _PhCommon:
    """
    This Class is mainly creates to avoid 'circular import' scenarios.

    Sample Error:
    ImportError: cannot import name 'PhUtil' from partially initialized module 'play_helpers.ph_util' (most likely due to a circular import) (D:\Other\Github_Self\playhelpers\play_helpers\ph_util.py)
    """

    @classmethod
    def get_key_value_pair(cls, key, value, sep=PhConstants.SEPERATOR_ONE_LINE, dic_format=False, print_also=False,
                           log=None, user_friendly_key=True, pair_is_must=False, length_needed=False):
        """

        :param length_needed:
        :param key:
        :param value:
        :param sep:
        :param dic_format:
        :param print_also:
        :param log:
        :param user_friendly_key:
        :param pair_is_must:
        :return:
        """
        print_or_log = log.info if log else print
        if key is None:
            return None
        if value is None:
            if pair_is_must is True:
                return None
            value = ''
        if length_needed:
            if PhKeys.LENGTH not in key:
                key = f'{key}_{PhKeys.LENGTH}'
            value = len(str(value))
        str_data = f'{cls.get_user_friendly_name(key) if user_friendly_key else key}{sep}{value}'
        if print_also:
            print_or_log(str_data)
        if dic_format:
            return {key: value}
        return str_data

    @classmethod
    def get_user_friendly_name(cls, python_variable_name):
        """

        :param python_variable_name:
        :return:
        """
        temp_data = re.sub(r'[_]', repl=' ', string=python_variable_name)
        return temp_data.title()



class PhFileExtensions:
    ALL_EXT = '.*'
    SEP = '.'
    TXT = '.txt'
    DOC = '.doc'
    DOCX = '.docx'
    PDF = '.pdf'
    ASN = '.asn'
    ASN1 = '.asn1'
    BASE_64 = '.base64'
    HEX = '.hex'
    YML = '.yml'
    YAML = '.yaml'
    JSON = '.json'
    IN = '.in'
    OUT = '.out'
    CSV = '.csv'
    XLSX = '.xlsx'
    XLS = '.xls'
    PNG = '.png'
    SVG = '.svg'
    JPG = '.jpg'
    JPEG = '.jpeg'
    BKP = '.bkp'
    TMP = '.tmp'
    PY = '.py'
    JAVA = '.java'
    _7Z = '.7z'
    ZIP = '.zip'
    HASH = '.hash'
    FUNCTION_SPECIFIC = '.function_specific'
    ASN_ORG = '.asn_org'
    ASN_MANUALLY_CORRECTED = '.asn_manually_corrected'
    ASN_AUTO_CORRECTED = '.asn_auto_corrected'
    ASN_FORMATTED = '.asn_formatted'


class PhFileExtensionsGroups:
    ARCHIVE_OUTPUT_FORMATS_SUPPORTED = [
        PhFileExtensions.ZIP,
    ]



from play_helpers.ph_keys import PhKeys


class PhVariables:
    #####################
    # asn1Play
    INPUT_FORMAT = f'${PhKeys.INPUT_FORMAT}'
    OUTPUT_FORMAT = f'${PhKeys.OUTPUT_FORMAT}'
    ASN1_ELEMENT = f'${PhKeys.ASN1_ELEMENT}'
    TLV_PARSING_OF_OUTPUT = f'${PhKeys.TLV_PARSING_OF_OUTPUT}'
    RE_PARSE_OUTPUT = f'${PhKeys.RE_PARSE_OUTPUT}'

    # asn1Play Web

    # asn1

    # asn1 Web

    # tlvPlay
    LENGTH_IN_DECIMAL = f'${PhKeys.LENGTH_IN_DECIMAL}'
    VALUE_IN_ASCII = f'${PhKeys.VALUE_IN_ASCII}'
    ONE_LINER = f'${PhKeys.ONE_LINER}'
    NON_TLV_NEIGHBOR = f'${PhKeys.NON_TLV_NEIGHBOR}'

    # qrPlay
    # OUTPUT_FORMAT = f'${PhKeys.OUTPUT_FORMAT}'
    SIZE = f'${PhKeys.SIZE}'
    QR_CODE_VERSION = f'${PhKeys.QR_CODE_VERSION}'
    SPLIT_QRS = f'${PhKeys.SPLIT_QRS}'
    DECORATE_QR = f'${PhKeys.DECORATE_QR}'
    LABEL = f'${PhKeys.LABEL}'
    LABEL_POSITION = f'${PhKeys.LABEL_POSITION}'

    # qrPlay Web

    # certPlay
    INPUT_FORMAT = f'${PhKeys.INPUT_FORMAT}'
    URL_TIME_OUT = f'${PhKeys.URL_TIME_OUT}'
    URL_PRE_ACCESS = f'${PhKeys.URL_PRE_ACCESS}'
    URL_CERT_FETCH_ONLY = f'${PhKeys.URL_CERT_FETCH_ONLY}'
    URL_ALL_CERTS = f'${PhKeys.URL_ALL_CERTS}'

    # certPlay Web

    # dataPlay
    CONTENT_MAPPINGS = f'${PhKeys.CONTENT_MAPPINGS}'
    NAME_MAPPINGS = f'${PhKeys.NAME_MAPPINGS}'

    #####################
    VERSION = f'${PhKeys.VERSION}'
    REMARKS = f'${PhKeys.REMARKS}'
    ITEM_INDEX = '$ITEM_INDEX'
    KEY_NAME = '$KEY_NAME'
